@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dbfopen.c,v 1.6 2003/12/01 20:52:00 strk Exp $
+ * $Id: dbfopen.c,v 1.8 2006/01/16 10:42:57 strk Exp $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of .dbf access API documented in dbf_api.html.
@@ -34,6 +34,12 @@
  ******************************************************************************
  *
  * $Log: dbfopen.c,v $
+ * Revision 1.8  2006/01/16 10:42:57  strk
+ * Added support for Bool and Date DBF<=>PGIS mapping
+ *
+ * Revision 1.7  2006/01/09 16:40:16  strk
+ * ISO C90 comments, signedness mismatch fixes
+ *
  * Revision 1.6  2003/12/01 20:52:00  strk
  * shapelib put in sync with gdal cvs
  *
@@ -198,7 +204,7 @@
  */
 
 static char rcsid[] = 
-  "$Id: dbfopen.c,v 1.6 2003/12/01 20:52:00 strk Exp $";
+  "$Id: dbfopen.c,v 1.8 2006/01/16 10:42:57 strk Exp $";
 
 #include "shapefil.h"
 
@@ -590,6 +596,7 @@ DBFCreate( const char * pszFilename )
     psDBF->nFields = 0;
     psDBF->nRecordLength = 1;
     psDBF->nHeaderLength = 33;
+    psDBF->bUpdated = FALSE;
     
     psDBF->panFieldOffset = NULL;
     psDBF->panFieldSize = NULL;
@@ -666,6 +673,8 @@ DBFAddField(DBFHandle psDBF, const char * pszFieldName,
         psDBF->pachFieldType[psDBF->nFields-1] = 'L';
     else if( eType == FTString )
         psDBF->pachFieldType[psDBF->nFields-1] = 'C';
+    else if( eType == FTDate )
+        psDBF->pachFieldType[psDBF->nFields-1] = 'D';
     else
         psDBF->pachFieldType[psDBF->nFields-1] = 'N';
 
@@ -981,9 +990,11 @@ DBFGetFieldInfo( DBFHandle psDBF, int iField, char * pszFieldName,
     if ( psDBF->pachFieldType[iField] == 'L' )
 	return( FTLogical);
 
+    else if( psDBF->pachFieldType[iField] == 'D'  )
+	return ( FTDate );
+
     else if( psDBF->pachFieldType[iField] == 'N' 
-             || psDBF->pachFieldType[iField] == 'F'
-             || psDBF->pachFieldType[iField] == 'D' )
+             || psDBF->pachFieldType[iField] == 'F' )
     {
 	if( psDBF->panFieldDecimals[iField] > 0 )
 	    return( FTDouble );
@@ -1173,7 +1184,7 @@ static int DBFWriteAttribute(DBFHandle psDBF, int hEntity, int iField,
 /************************************************************************/
 
 int DBFWriteAttributeDirectly(DBFHandle psDBF, int hEntity, int iField,
-                              void * pValue )
+                              const void * pValue )
 
 {
     int	       	nRecordOffset, i, j;
