@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql.c,v 1.88 2005/04/14 12:58:59 strk Exp $
+ * $Id: shp2pgsql.c,v 1.106 2006/01/16 10:42:58 strk Exp $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -22,6 +22,7 @@
  *
  **********************************************************************/
 
+#include "../config.h"
 #include "shapefil.h"
 #include <stdio.h>
 #include <string.h>
@@ -32,6 +33,10 @@
 #include <unistd.h>
 #include <errno.h>
 #include "getopt.h"
+#ifdef HAVE_ICONV_H
+#include <iconv.h>
+#endif
+
 
 #define	POINTTYPE	1
 #define	LINETYPE	2
@@ -44,21 +49,29 @@
 #define WKBZOFFSET 0x80000000
 #define WKBMOFFSET 0x40000000
 
-//#define DEBUG 1
+/*#define DEBUG 1 */
 
 typedef struct {double x, y, z, m;} Point;
 
 typedef struct Ring {
-	Point *list;	//list of points
+	Point *list;		/* list of points */
 	struct Ring  *next;
-	int n;	//number of points in list
-	unsigned int linked; // number of "next" rings
+	int n;			/* number of points in list */
+	unsigned int linked; 	/* number of "next" rings */
 } Ring;
 
+/* Values for null_policy global */
+enum {
+	insert_null,
+	skip_null,
+	abort_on_null
+};
+
 /* globals */
-int	dump_format = 0; //0=insert statements, 1 = dump
+int	dump_format = 0; /* 0=insert statements, 1 = dump */
 int	quoteidentifiers = 0;
 int	forceint4 = 0;
+int	createindex = 0;
 char    opt = ' ';
 char    *col_names = NULL;
 char	*pgtype;
@@ -66,10 +79,11 @@ int	istypeM = 0;
 int	pgdims;
 unsigned int wkbtype;
 char  	*shp_file = NULL;
-int	hwgeom = 0; // old (hwgeom) mode
+int	hwgeom = 0; /* old (hwgeom) mode */
 #ifdef USE_ICONV
 char	*encoding=NULL;
 #endif
+int null_policy = insert_null;
 
 DBFFieldType *types;	/* Fields type, width and precision */
 SHPHandle  hSHPHandle;
@@ -90,6 +104,7 @@ char *protect_quotes_string(char *str);
 int PIP( Point P, Point* V, int n );
 void *safe_malloc(size_t size);
 void CreateTable(void);
+void CreateIndex(void);
 void usage(char *me, int exitcode);
 void InsertPoint(void);
 void InsertPointWKT(void);
@@ -109,13 +124,12 @@ void ReleasePolygons(Ring **polys, int npolys);
 void DropTable(char *schema, char *table, char *geom);
 void GetFieldsSpec(void);
 void LoadData(void);
-void UpdateSequence(void);
 void OpenShape(void);
 void LowerCase(char *s);
 void Cleanup(void);
 
 
-// WKB
+/* WKB */
 static char getEndianByte(void);
 static void print_wkb_bytes(unsigned char* ptr, unsigned int cnt, size_t size);
 static void print_wkb_byte(unsigned char val);
@@ -123,7 +137,7 @@ static void print_wkb_int(int val);
 static void print_wkb_double(double val);
 
 static char rcsid[] =
-  "$Id: shp2pgsql.c,v 1.88 2005/04/14 12:58:59 strk Exp $";
+  "$Id: shp2pgsql.c,v 1.106 2006/01/16 10:42:58 strk Exp $";
 
 void *safe_malloc(size_t size)
 {
@@ -141,12 +155,14 @@ void *safe_malloc(size_t size)
 char *
 make_good_string(char *str)
 {
-	//find all the tabs and make them \<tab>s
-	//
-	// 1. find # of tabs
-	// 2. make new string 
-	//
-	// we dont escape already escaped tabs
+	/*
+	 * find all the tabs and make them \<tab>s
+	 *
+	 * 1. find # of tabs
+	 * 2. make new string 
+	 *
+	 * we dont escape already escaped tabs
+	 */
 
 	char *result;
 	char *ptr, *optr;
@@ -172,7 +188,7 @@ make_good_string(char *str)
 
 	if (toescape == 0) return str;
 
-	size = ptr-str+toescape;
+	size = ptr-str+toescape+1;
 
 	result = calloc(1, size);
 
@@ -185,7 +201,7 @@ make_good_string(char *str)
 	*optr='\0';
 
 #ifdef USE_ICONV
-	free(str);
+	if ( encoding ) free(str);
 #endif
 
 	return result;
@@ -195,11 +211,13 @@ make_good_string(char *str)
 char *
 protect_quotes_string(char *str)
 {
-	//find all quotes and make them \quotes
-	//find all '\' and make them '\\'
-	// 
-	// 1. find # of characters
-	// 2. make new string 
+	/*
+	 * find all quotes and make them \quotes
+	 * find all '\' and make them '\\'
+	 * 	 
+	 * 1. find # of characters
+	 * 2. make new string 
+	 */
 
 	char	*result;
 	char	*ptr, *optr;
@@ -225,7 +243,7 @@ protect_quotes_string(char *str)
 
 	if (toescape == 0) return str;
 	
-	size = ptr-str+toescape;
+	size = ptr-str+toescape+1;
 
 	result = calloc(1, size);
 
@@ -238,7 +256,7 @@ protect_quotes_string(char *str)
 	*optr='\0';
 
 #ifdef USE_ICONV
-	free(str);
+	if ( encoding ) free(str);
 #endif
 
 	return result;
@@ -246,31 +264,33 @@ protect_quotes_string(char *str)
 
 
 
-// PIP(): crossing number test for a point in a polygon
-//      input:   P = a point,
-//               V[] = vertex points of a polygon V[n+1] with V[n]=V[0]
-//      returns: 0 = outside, 1 = inside
+/*
+ * PIP(): crossing number test for a point in a polygon
+ *      input:   P = a point,
+ *               V[] = vertex points of a polygon V[n+1] with V[n]=V[0]
+ *      returns: 0 = outside, 1 = inside
+ */
 int
 PIP( Point P, Point* V, int n )
 {
-	int cn = 0;    // the crossing number counter
+	int cn = 0;    /* the crossing number counter */
 	int i;
 
-    // loop through all edges of the polygon
-    for (i=0; i<n-1; i++) {    // edge from V[i] to V[i+1]
-       if (((V[i].y <= P.y) && (V[i+1].y > P.y))    // an upward crossing
-        || ((V[i].y > P.y) && (V[i+1].y <= P.y))) { // a downward crossing
+    /* loop through all edges of the polygon */
+    for (i=0; i<n-1; i++) {    /* edge from V[i] to V[i+1] */
+       if (((V[i].y <= P.y) && (V[i+1].y > P.y))    /* an upward crossing */
+        || ((V[i].y > P.y) && (V[i+1].y <= P.y))) { /* a downward crossing */
             double vt = (float)(P.y - V[i].y) / (V[i+1].y - V[i].y);
-            if (P.x < V[i].x + vt * (V[i+1].x - V[i].x)) // P.x < intersect
-                ++cn;   // a valid crossing of y=P.y right of P.x
+            if (P.x < V[i].x + vt * (V[i+1].x - V[i].x)) /* P.x < intersect */
+                ++cn;   /* a valid crossing of y=P.y right of P.x */
         }
     }
-    return (cn&1);    // 0 if even (out), and 1 if odd (in)
+    return (cn&1);    /* 0 if even (out), and 1 if odd (in) */
 
 }
 
 
-//Insert the attributes from the correct row of dbf file
+/*Insert the attributes from the correct row of dbf file */
 
 int
 Insert_attributes(DBFHandle hDBFHandle, int row)
@@ -308,11 +328,14 @@ Insert_attributes(DBFHandle hDBFHandle, int row)
                   fprintf(stderr, "Warning: field %d name trucated\n", i);
                   val[1023] = '\0';
                }
-	       // pg_atoi() does not do this
+	       /* pg_atoi() does not do this */
 	       if ( val[0] == '\0' ) { val[0] = '0'; val[1] = '\0'; }
+	       if ( val[strlen(val)-1] == '.' ) val[strlen(val)-1] = '\0';
                break;
+
             case FTString:
             case FTLogical:
+	    case FTDate:
                if ( -1 == snprintf(val, 1024, "%s",
                      DBFReadStringAttribute(hDBFHandle, row, i)) )
                {
@@ -320,6 +343,7 @@ Insert_attributes(DBFHandle hDBFHandle, int row)
                   val[1023] = '\0';
                }
                break;
+
             default:
                fprintf(stderr,
                   "Error: field %d has invalid or unknown field type (%d)\n",
@@ -345,9 +369,10 @@ Insert_attributes(DBFHandle hDBFHandle, int row)
 
 
 
-// main()     
-//see description at the top of this file
-
+/*
+ * main()     
+ * see description at the top of this file
+ */
 int
 main (int ARGC, char **ARGV)
 {
@@ -375,7 +400,7 @@ main (int ARGC, char **ARGV)
 	{
 		printf("SET CLIENT_ENCODING TO UTF8;\n");
 	}
-#endif // defined USE_ICONV
+#endif /* defined USE_ICONV */
 
 	/*
 	 * Drop table if requested
@@ -395,16 +420,21 @@ main (int ARGC, char **ARGV)
 	 */
 	if(opt != 'a') CreateTable();
 
+	/*
+	 * Create GiST index if requested
+	 */
+	if(createindex) CreateIndex();
+
 	/*
 	 * Generate INSERT or COPY lines
 	 */
 	if(opt != 'p') LoadData();
 
-	printf("END;\n"); // End the last transaction
+	printf("END;\n"); /* End the last transaction */
 
 
-	return(1);
-}//end main()
+	return 0; 
+}/*end main() */
 
 void
 LowerCase(char *s)
@@ -423,6 +453,7 @@ void
 OpenShape(void)
 {
 	int j;
+	SHPObject *obj=NULL;
 
 	hSHPHandle = SHPOpen( shp_file, "rb" );
 	if (hSHPHandle == NULL) {
@@ -437,37 +468,25 @@ OpenShape(void)
 	}
 	SHPGetInfo(hSHPHandle, &num_entities, &shpfiletype, NULL, NULL);
 
-	/* Check we have at least a not-null geometry */
-	j=num_entities;
-	while( ( obj == NULL || obj->nVertices == 0 ) && j--)
-		obj = SHPReadObject(hSHPHandle,j);
-	if ( obj == NULL) 
+	if ( null_policy == abort_on_null )
 	{
-		fprintf(stderr, "Shapefile contains %d NULL object(s)\n",
-			num_entities);
-		exit(-1);
-	}
-
-}
-
-/*
- * Not used
- */
-void
-UpdateSequence(void)
-{
-	if ( schema )
-	{
-		if(num_entities>1){
-			printf("SELECT setval('\"%s\".\"%s_gid_seq\"', %i, true);\n", schema, table, num_entities-1);
-		}
-	}
-	else
-	{
-		if(num_entities>1){
-			printf("SELECT setval('\"%s_gid_seq\"', %i, true);\n", table, num_entities-1);
+		for (j=0; j<num_entities; j++)
+		{
+			obj = SHPReadObject(hSHPHandle,j);
+			if ( ! obj )
+			{
+		fprintf(stderr, "Error reading shape object %d\n", j);
+		exit(1);
+			}
+			if ( obj->nVertices == 0 )
+			{
+		fprintf(stderr, "Empty geometries found, aborted.\n");
+		exit(1);
+			}
+			SHPDestroyObject(obj);
 		}
 	}
+
 }
 
 void
@@ -498,60 +517,64 @@ CreateTable(void)
 		field_width = widths[j];
 		field_precision = precisions[j];
 
-		printf(", \"%s\" ", field_names[j]);
+		printf(",\n\"%s\" ", field_names[j]);
 
-		if(hDBFHandle->pachFieldType[j] == 'D' ) /* Date field */
+		if(type == FTString)
 		{
-			printf ("varchar(8)");//date data-type is not supported in API so check for it explicity before the api call.
+			/* use DBF attribute size as maximum width */
+			printf ("varchar(%d)", field_width);
 		}
-			
-		else
+		else if (type == FTDate)
 		{
-
-			if(type == FTString)
+			printf ("date");
+		}
+		else if (type == FTInteger)
+		{
+			if ( forceint4 )
 			{
-				printf ("varchar");
+				printf ("int4");
 			}
-			else if(type == FTInteger)
+			else if  ( field_width <= 5 )
 			{
-				if ( forceint4 || field_width <= 9 )
-				{
-					printf ("int4");
-				}
-				else if( field_width > 18 )
-				{
-					printf("numeric(%d,0)",
-						field_width);
-				}
-				else 
-				{
-					printf ("int8");
-				}
+				printf ("int2");
 			}
-			else if(type == FTDouble)
+			else if  ( field_width <= 10 )
 			{
-				if( field_width > 18 )
-				{
-					printf ("numeric");
-				}
-				else
-				{
-					printf ("float8");
-				}
+				printf ("int4");
 			}
-			else if(type == FTLogical)
+			else if  ( field_width <= 19 )
 			{
-				printf ("boolean");
+				printf ("int8");
+			}
+			else 
+			{
+				printf("numeric(%d,0)",
+					field_width);
+			}
+		}
+		else if(type == FTDouble)
+		{
+			if( field_width > 18 )
+			{
+				printf ("numeric");
 			}
 			else
 			{
-				printf ("Invalid type in DBF file");
+				printf ("float8");
 			}
-		}	
+		}
+		else if(type == FTLogical)
+		{
+			printf ("boolean");
+		}
+		else
+		{
+			printf ("Invalid type in DBF file");
+		}
 	}
 	printf (");\n");
 
-	//create the geometry column with an addgeometry call to dave's function
+	/* Create the geometry column with an addgeometry call */
 	if ( schema )
 	{
 		printf("SELECT AddGeometryColumn('%s','%s','%s','%s',",
@@ -566,6 +589,22 @@ CreateTable(void)
 	printf("'%s',%d);\n", pgtype, pgdims);
 }
 
+void
+CreateIndex(void)
+{
+	/* 
+	 * Create gist index
+	 */
+	if ( schema )
+	{
+		printf("CREATE INDEX \"%s_%s_gist\" ON \"%s\".\"%s\" using gist (\"%s\" gist_geometry_ops);\n", table, geom, schema, table, geom);
+	}
+	else
+	{
+		printf("CREATE INDEX \"%s_%s_gist\" ON \"%s\" using gist (\"%s\" gist_geometry_ops);\n", table, geom, table, geom);
+	}
+}
+
 void
 LoadData(void)
 {
@@ -592,7 +631,7 @@ LoadData(void)
 	 **************************************************************/
 	for (j=0; j<num_entities; j++)
 	{
-		//wrap a transaction block around each 250 inserts...
+		/*wrap a transaction block around each 250 inserts... */
 		if ( ! dump_format )
 		{
 			if (trans == 250) {
@@ -602,10 +641,21 @@ LoadData(void)
 			}
 		}
 		trans++;
-		// transaction stuff done
+		/* transaction stuff done */
 
-		//open the next object
+		/*open the next object */
 		obj = SHPReadObject(hSHPHandle,j);
+		if ( ! obj )
+		{
+			fprintf(stderr, "Error reading shape object %d\n", j);
+			exit(1);
+		}
+
+		if ( null_policy == skip_null && obj->nVertices == 0 )
+		{
+			SHPDestroyObject(obj);
+			continue;
+		}
 
 		if (!dump_format)
 		{
@@ -622,7 +672,7 @@ LoadData(void)
 		}
 		Insert_attributes(hDBFHandle,j);
 
-		// ---------- NULL SHAPE -----------------
+		/* ---------- NULL SHAPE ----------------- */
 		if (obj->nVertices == 0)
 		{
 			if (dump_format) printf("\\N\n");
@@ -670,7 +720,7 @@ LoadData(void)
 		
 		SHPDestroyObject(obj);	
 
-	} // END of MAIN SHAPE OBJECT LOOP
+	} /* END of MAIN SHAPE OBJECT LOOP */
 
 
 	if ((dump_format) ) {
@@ -707,19 +757,23 @@ usage(char *me, int exitcode)
 	fprintf(stderr, "\n");
 	fprintf(stderr, "  -i  Use int4 type for all integer dbf fields.\n");
 	fprintf(stderr, "\n");
+	fprintf(stderr, "  -I  Create a GiST index on the geometry column.\n");
+	fprintf(stderr, "\n");
 	fprintf(stderr, "  -w  Use wkt format (for postgis-0.x support - drops M - drifts coordinates).\n");
 #ifdef USE_ICONV
 	fprintf(stderr, "\n");
-	fprintf(stderr, "  -W <ENCODING_NAME> Specify the character encoding of Shape's\n");
+	fprintf(stderr, "  -W <encoding> Specify the character encoding of Shape's\n");
 	fprintf(stderr, "     attribute column. (default : \"ASCII\")\n");
 #endif
+	fprintf(stderr, "\n");
+	fprintf(stderr, "  -N <policy> Specify NULL geometries handling policy (insert,skip,abort)\n");
 	exit (exitcode);
 }
 
 void
 InsertLineString(int id)
 {
-	int pi; // part index
+	int pi; /* part index */
 	unsigned int subtype = LINETYPE | (wkbtype&WKBZOFFSET) | 
 		(wkbtype&WKBMOFFSET);
 
@@ -732,7 +786,6 @@ InsertLineString(int id)
 		if (dump_format) printf("\\N\n");
 		else printf("NULL);\n");
 
-		SHPDestroyObject(obj);
 		return;
 	}
 
@@ -745,14 +798,14 @@ InsertLineString(int id)
 
 	for (pi=0; pi<obj->nParts; pi++)
 	{
-		int vi; // vertex index
-		int vs; // start vertex
-		int ve; // end vertex
+		int vi; /* vertex index */
+		int vs; /* start vertex */
+		int ve; /* end vertex */
 
 		print_wkb_byte(getEndianByte());
 		print_wkb_int(subtype);
 
-		// Set start and end vertexes
+		/* Set start and end vertexes */
 		if ( pi==obj->nParts-1 ) ve = obj->nVertices;
 		else ve = obj->panPartStart[pi+1];
 		vs = obj->panPartStart[pi];
@@ -776,7 +829,7 @@ InsertLineString(int id)
 void
 InsertLineStringWKT(int id)
 {
-	int pi; // part index
+	int pi; /* part index */
 
 	/* Invalid (MULTI)Linestring */
 	if ( obj->nVertices < 2 )
@@ -787,7 +840,6 @@ InsertLineStringWKT(int id)
 		if (dump_format) printf("\\N\n");
 		else printf("NULL);\n");
 
-		SHPDestroyObject(obj);
 		return;
 	}
 
@@ -796,14 +848,14 @@ InsertLineStringWKT(int id)
 
 	for (pi=0; pi<obj->nParts; pi++)
 	{
-		int vi; // vertex index
-		int vs; // start vertex
-		int ve; // end vertex
+		int vi; /* vertex index */
+		int vs; /* start vertex */
+		int ve; /* end vertex */
 
 		if (pi) printf(",");
 		printf("(");
 
-		// Set start and end vertexes
+		/* Set start and end vertexes */
 		if ( pi==obj->nParts-1 ) ve = obj->nVertices;
 		else ve = obj->panPartStart[pi+1];
 		vs = obj->panPartStart[pi];
@@ -829,11 +881,11 @@ InsertLineStringWKT(int id)
 int
 FindPolygons(SHPObject *obj, Ring ***Out)
 {
-	Ring **Outer;    // Pointers to Outer rings
-	int out_index=0; // Count of Outer rings
-	Ring **Inner;    // Pointers to Inner rings
-	int in_index=0;  // Count of Inner rings
-	int pi; // part index
+	Ring **Outer;    /* Pointers to Outer rings */
+	int out_index=0; /* Count of Outer rings */
+	Ring **Inner;    /* Pointers to Inner rings */
+	int in_index=0;  /* Count of Inner rings */
+	int pi; /* part index */
 
 #ifdef DEBUG
 	static int call = -1;
@@ -843,39 +895,39 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 		call, obj->nParts);
 #endif
 
-	// Allocate initial memory
+	/* Allocate initial memory */
 	Outer = (Ring**)malloc(sizeof(Ring*)*obj->nParts);
 	Inner = (Ring**)malloc(sizeof(Ring*)*obj->nParts);
 
-	// Iterate over rings dividing in Outers and Inners
+	/* Iterate over rings dividing in Outers and Inners */
 	for (pi=0; pi<obj->nParts; pi++)
 	{
-		int vi; // vertex index
-		int vs; // start index
-		int ve; // end index
-		int nv; // number of vertex
+		int vi; /* vertex index */
+		int vs; /* start index */
+		int ve; /* end index */
+		int nv; /* number of vertex */
 		double area = 0.0;
 		Ring *ring;
 
-		// Set start and end vertexes
+		/* Set start and end vertexes */
 		if ( pi==obj->nParts-1 ) ve = obj->nVertices;
 		else ve = obj->panPartStart[pi+1];
 		vs = obj->panPartStart[pi];
 
-		// Compute number of vertexes
+		/* Compute number of vertexes */
 		nv = ve-vs;
 
-		// Allocate memory for a ring
+		/* Allocate memory for a ring */
 		ring = (Ring*)malloc(sizeof(Ring));
 		ring->list = (Point*)malloc(sizeof(Point)*nv);
 		ring->n = nv;
 		ring->next = NULL;
 		ring->linked = 0;
 
-		// Iterate over ring vertexes
+		/* Iterate over ring vertexes */
 		for ( vi=vs; vi<ve; vi++)
 		{
-			int vn = vi+1; // next vertex for area
+			int vn = vi+1; /* next vertex for area */
 			if ( vn==ve ) vn = vs;
 
 			ring->list[vi-vs].x = obj->padfX[vi];
@@ -887,19 +939,19 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 				(obj->padfY[vi] * obj->padfX[vn]); 
 		}
 
-		// Close the ring with first vertex 
-		//ring->list[vi].x = obj->padfX[vs];
-		//ring->list[vi].y = obj->padfY[vs];
-		//ring->list[vi].z = obj->padfZ[vs];
-		//ring->list[vi].m = obj->padfM[vs];
+		/* Close the ring with first vertex  */
+		/*ring->list[vi].x = obj->padfX[vs]; */
+		/*ring->list[vi].y = obj->padfY[vs]; */
+		/*ring->list[vi].z = obj->padfZ[vs]; */
+		/*ring->list[vi].m = obj->padfM[vs]; */
 
-		// Clockwise (or single-part). It's an Outer Ring !
+		/* Clockwise (or single-part). It's an Outer Ring ! */
 		if(area < 0.0 || obj->nParts ==1) {
 			Outer[out_index] = ring;
 			out_index++;
 		}
 
-		// Counterclockwise. It's an Inner Ring !
+		/* Counterclockwise. It's an Inner Ring ! */
 		else {
 			Inner[in_index] = ring;
 			in_index++;
@@ -911,8 +963,8 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 		call, out_index, in_index);
 #endif
 
-	// Put the inner rings into the list of the outer rings
-	// of which they are within
+	/* Put the inner rings into the list of the outer rings */
+	/* of which they are within */
 	for(pi=0; pi<in_index; pi++)
 	{
 		Point pt,pt2;
@@ -935,7 +987,7 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 				outer = Outer[i];
 				break;
 			}
-			//fprintf(stderr, "!PIP %s\nOUTE %s\n", dump_ring(inner), dump_ring(Outer[i]));
+			/*fprintf(stderr, "!PIP %s\nOUTE %s\n", dump_ring(inner), dump_ring(Outer[i])); */
 		}
 
 		if ( outer )
@@ -946,8 +998,8 @@ FindPolygons(SHPObject *obj, Ring ***Out)
 		}
 		else
 		{
-			// The ring wasn't within any outer rings,
-			// assume it is a new outer ring.
+			/* The ring wasn't within any outer rings, */
+			/* assume it is a new outer ring. */
 #ifdef DEBUG
 			fprintf(stderr,
 				"FindPolygons[%d]: hole %d is orphan\n",
@@ -968,7 +1020,7 @@ void
 ReleasePolygons(Ring **polys, int npolys)
 {
 	int pi;
-	// Release all memory
+	/* Release all memory */
 	for(pi=0; pi<npolys; pi++)
 	{
 		Ring *Poly, *temp;
@@ -982,9 +1034,9 @@ ReleasePolygons(Ring **polys, int npolys)
 	}
 }
 
-//This function basically deals with the polygon case.
-//it sorts the polys in order of outer,inner,inner, so that inners
-//always come after outers they are within 
+/*This function basically deals with the polygon case. */
+/*it sorts the polys in order of outer,inner,inner, so that inners */
+/*always come after outers they are within  */
 void
 InsertPolygon(void)
 {
@@ -992,7 +1044,7 @@ InsertPolygon(void)
 		(wkbtype&WKBMOFFSET);
 	Ring **Outer;
 	int out_index;
-	int pi; // part index
+	int pi; /* part index */
 
 	out_index = FindPolygons(obj, &Outer);
 
@@ -1001,9 +1053,9 @@ InsertPolygon(void)
 
 	print_wkb_byte(getEndianByte());
 	print_wkb_int(wkbtype);
-	print_wkb_int(out_index); // npolys
+	print_wkb_int(out_index); /* npolys */
 
-	// Write the coordinates
+	/* Write the coordinates */
 	for(pi=0; pi<out_index; pi++)
 	{
 		Ring *poly;
@@ -1012,13 +1064,13 @@ InsertPolygon(void)
 
 		print_wkb_byte(getEndianByte());
 		print_wkb_int(subtype);
-		print_wkb_int(poly->linked+1); // nrings
+		print_wkb_int(poly->linked+1); /* nrings */
 
 		while(poly)
 		{
-			int vi; // vertex index
+			int vi; /* vertex index */
 
-			print_wkb_int(poly->n); // npoints
+			print_wkb_int(poly->n); /* npoints */
 
 			for(vi=0; vi<poly->n; vi++)
 			{
@@ -1038,7 +1090,7 @@ InsertPolygon(void)
 	if (dump_format) printf("\n");
 	else printf("');\n");
 
-	// Release all memory
+	/* Release all memory */
 	ReleasePolygons(Outer, out_index);
 	free(Outer);
 }
@@ -1046,9 +1098,9 @@ InsertPolygon(void)
 void
 InsertPolygonWKT(void)
 {
-	Ring **Outer;    // Pointers to Outer rings
-	int out_index=0; // Count of Outer rings
-	int pi; // part index
+	Ring **Outer;    /* Pointers to Outer rings */
+	int out_index=0; /* Count of Outer rings */
+	int pi; /* part index */
 
 #ifdef DEBUG
 	static int call = -1;
@@ -1063,7 +1115,7 @@ InsertPolygonWKT(void)
 	if (dump_format) printf("SRID=%s;MULTIPOLYGON(",sr_id );
 	else printf("GeometryFromText('MULTIPOLYGON(");
 
-	// Write the coordinates
+	/* Write the coordinates */
 	for(pi=0; pi<out_index; pi++)
 	{
 		Ring *poly;
@@ -1075,7 +1127,7 @@ InsertPolygonWKT(void)
 
 		while(poly)
 		{
-			int vi; // vertex index
+			int vi; /* vertex index */
 
 			printf("(");
 			for(vi=0; vi<poly->n; vi++)
@@ -1100,7 +1152,7 @@ InsertPolygonWKT(void)
 	if (dump_format) printf(")\n");
 	else printf(")',%s) );\n",sr_id);
 
-	// Release all memory
+	/* Release all memory */
 	ReleasePolygons(Outer, out_index);
 	free(Outer);
 }
@@ -1126,8 +1178,8 @@ void
 InsertPointWKT(void)
 {
 	unsigned int u;
-	if (dump_format) printf("SRID=%s;POINT(",sr_id);
-	else printf("GeometryFromText('POINT (");
+	if (dump_format) printf("SRID=%s;%s(", sr_id, pgtype);
+	else printf("GeometryFromText('%s(", pgtype);
 
 	for (u=0;u<obj->nVertices; u++){
 		if (u>0) printf(",");
@@ -1176,7 +1228,7 @@ ParseCmdline(int ARGC, char **ARGV)
 	extern char *optarg;
 	extern int optind;
 
-	while ((c = getopt(ARGC, ARGV, "kcdapDs:g:iW:w")) != EOF){
+	while ((c = getopt(ARGC, ARGV, "kcdapDs:g:iW:wIN:")) != EOF){
                switch (c) {
                case 'c':
                     if (opt == ' ')
@@ -1217,14 +1269,36 @@ ParseCmdline(int ARGC, char **ARGV)
                case 'i':
                     forceint4 = 1;
                     break;
+               case 'I':
+                    createindex = 1;
+                    break;
                case 'w':
                     hwgeom = 1;
                     break;
-#ifdef USE_ICONV
 		case 'W':
+#ifdef USE_ICONV
                     encoding = optarg;
-                    break;
+#else
+		    fprintf(stderr, "WARNING: the -W switch will have no effect. UTF8 disabled at compile time\n");
 #endif
+                    break;
+		case 'N':
+			switch (optarg[0])
+			{	
+				case 'a':
+					null_policy = abort_on_null;
+					break;
+				case 'i':
+					null_policy = insert_null;
+					break;
+				case 's':
+					null_policy = skip_null;
+					break;
+				default:
+					fprintf(stderr, "Unsupported NULL geometry handling policy.\nValid policies: insert, skip, abort\n");
+					exit(1);
+			}
+                    break;
                case '?':
                default:              
 		return 0;
@@ -1279,27 +1353,27 @@ SetPgType(void)
 {
 	switch(shpfiletype)
 	{
-		case SHPT_POINT: // Point
+		case SHPT_POINT: /* Point */
 			pgtype = "POINT";
 			wkbtype = POINTTYPE;
 			pgdims = 2;
 			break;
-		case SHPT_ARC: // PolyLine
+		case SHPT_ARC: /* PolyLine */
 			pgtype = "MULTILINESTRING";
 			wkbtype = MULTILINETYPE ;
 			pgdims = 2;
 			break;
-		case SHPT_POLYGON: // Polygon
+		case SHPT_POLYGON: /* Polygon */
 			pgtype = "MULTIPOLYGON";
 			wkbtype = MULTIPOLYGONTYPE;
 			pgdims = 2;
 			break;
-		case SHPT_MULTIPOINT: // MultiPoint
+		case SHPT_MULTIPOINT: /* MultiPoint */
 			pgtype = "MULTIPOINT";
 			wkbtype = MULTIPOINTTYPE;
 			pgdims = 2;
 			break;
-		case SHPT_POINTM: // PointM
+		case SHPT_POINTM: /* PointM */
 			wkbtype = POINTTYPE | WKBMOFFSET;
 			if ( ! hwgeom ) {
 				pgtype = "POINTM";
@@ -1310,7 +1384,7 @@ SetPgType(void)
 				pgdims = 2;
 			}
 			break;
-		case SHPT_ARCM: // PolyLineM
+		case SHPT_ARCM: /* PolyLineM */
 			wkbtype = MULTILINETYPE | WKBMOFFSET;
 			if ( ! hwgeom ) {
 				pgtype = "MULTILINESTRINGM";
@@ -1321,7 +1395,7 @@ SetPgType(void)
 				pgdims = 2;
 			}
 			break;
-		case SHPT_POLYGONM: // PolygonM
+		case SHPT_POLYGONM: /* PolygonM */
 			wkbtype = MULTIPOLYGONTYPE | WKBMOFFSET;
 			if ( ! hwgeom ) {
 				pgtype = "MULTIPOLYGONM";
@@ -1332,7 +1406,7 @@ SetPgType(void)
 				pgdims = 2;
 			}
 			break;
-		case SHPT_MULTIPOINTM: // MultiPointM
+		case SHPT_MULTIPOINTM: /* MultiPointM */
 			wkbtype = MULTIPOINTTYPE | WKBMOFFSET;
 			if ( ! hwgeom ) {
 				pgtype = "MULTIPOINTM";
@@ -1343,25 +1417,25 @@ SetPgType(void)
 				pgdims = 2;
 			}
 			break;
-		case SHPT_POINTZ: // PointZ
+		case SHPT_POINTZ: /* PointZ */
 			wkbtype = POINTTYPE | WKBMOFFSET | WKBZOFFSET;
 			pgtype = "POINT";
 			if ( ! hwgeom ) pgdims = 4;
 			else pgdims = 3;
 			break;
-		case SHPT_ARCZ: // PolyLineZ
+		case SHPT_ARCZ: /* PolyLineZ */
 			pgtype = "MULTILINESTRING";
 			wkbtype = MULTILINETYPE | WKBZOFFSET | WKBMOFFSET;
 			if ( ! hwgeom ) pgdims = 4;
 			else pgdims = 3;
 			break;
-		case SHPT_POLYGONZ: // MultiPolygonZ
+		case SHPT_POLYGONZ: /* MultiPolygonZ */
 			pgtype = "MULTIPOLYGON";
 			wkbtype = MULTIPOLYGONTYPE | WKBZOFFSET | WKBMOFFSET;
 			if ( ! hwgeom ) pgdims = 4;
 			else pgdims = 3;
 			break;
-		case SHPT_MULTIPOINTZ: // MultiPointZ
+		case SHPT_MULTIPOINTZ: /* MultiPointZ */
 			pgtype = "MULTIPOINT";
 			wkbtype = MULTIPOINTTYPE | WKBZOFFSET | WKBMOFFSET;
 			if ( ! hwgeom ) pgdims = 4;
@@ -1382,27 +1456,28 @@ dump_ring(Ring *ring)
 {
 	char *buf = malloc(256*ring->n);
 	int i;
+
+	buf[0] = '\0';
 	for (i=0; i<ring->n; i++)
 	{
-		if (i) sprintf(buf, "%s,", buf);
-		sprintf(buf, "%s%g %g",
-			buf,
+		if (i) strcat(buf, ",");
+		sprintf(buf+strlen(buf), "%g %g",
 			ring->list[i].x,
 			ring->list[i].y);
 	}
 	return buf;
 }
 
-//--------------- WKB handling 
+/*--------------- WKB handling  */
 
 static char outchr[]={"0123456789ABCDEF"};
 
-static int endian_check_int = 1; // dont modify this!!!
+static int endian_check_int = 1; /* dont modify this!!! */
 
 static char
 getEndianByte(void)
 {
-	// 0 = big endian, 1 = little endian
+	/* 0 = big endian, 1 = little endian */
 	if ( *((char *) &endian_check_int) ) return 1;
 	else return 0;
 }
@@ -1428,7 +1503,7 @@ print_wkb_int(int val)
 static void
 print_wkb_bytes(unsigned char *ptr, unsigned int cnt, size_t size)
 {
-	unsigned int bc; // byte count
+	unsigned int bc; /* byte count */
 	static char buf[256];
 	char *bufp;
 
@@ -1448,24 +1523,25 @@ print_wkb_bytes(unsigned char *ptr, unsigned int cnt, size_t size)
 		}
 	}
 	*bufp = '\0';
-	//fprintf(stderr, "\nwkbbytes:%s\n", buf);
+	/*fprintf(stderr, "\nwkbbytes:%s\n", buf); */
 	printf("%s", buf);
 }
 
 void
 DropTable(char *schema, char *table, char *geom)
 {
-		//---------------Drop the table--------------------------
-		// TODO: if the table has more then one geometry column
-		// the DROP TABLE call will leave spurious records in
-		// geometry_columns. 
-		//
-		// If the geometry column in the table being dropped
-		// does not match 'the_geom' or the name specified with
-		// -g an error is returned by DropGeometryColumn. 
-		//
-		// The table to be dropped might not exist.
-		//
+		/*---------------Drop the table--------------------------
+		 * TODO: if the table has more then one geometry column
+		 * the DROP TABLE call will leave spurious records in
+		 * geometry_columns. 
+		 *
+		 * If the geometry column in the table being dropped
+		 * does not match 'the_geom' or the name specified with
+		 * -g an error is returned by DropGeometryColumn. 
+		 *
+		 * The table to be dropped might not exist.
+		 *
+		 */
 		if ( schema )
 		{
 			printf("SELECT DropGeometryColumn('%s','%s','%s');\n",
@@ -1483,11 +1559,19 @@ DropTable(char *schema, char *table, char *geom)
 void
 GetFieldsSpec(void)
 {
+/*
+ * Shapefile (dbf) field name are at most 10chars + 1 NULL.
+ * Postgresql field names are at most 63 bytes + 1 NULL.
+ */
+#define MAXFIELDNAMELEN 64
 	int field_precision, field_width;
 	int j, z;
-	char  name[64];
-	char  name2[64];
+	char  name[MAXFIELDNAMELEN];
+	char  name2[MAXFIELDNAMELEN];
 	DBFFieldType type = -1;
+#ifdef USE_ICONV
+	char *utf8str;
+#endif
 
 	num_fields = DBFGetFieldCount( hDBFHandle );
 	num_records = DBFGetRecordCount(hDBFHandle);
@@ -1495,45 +1579,42 @@ GetFieldsSpec(void)
 	types = (DBFFieldType *)malloc(num_fields*sizeof(int));
 	widths = malloc(num_fields*sizeof(int));
 	precisions = malloc(num_fields*sizeof(int));
-	col_names = malloc((num_fields+2) * sizeof(char) * 32);
+	col_names = malloc((num_fields+2) * sizeof(char) * MAXFIELDNAMELEN);
 	strcpy(col_names, "(" );
 
-	//fprintf(stderr, "Number of fields from DBF: %d\n", num_fields);
+	/*fprintf(stderr, "Number of fields from DBF: %d\n", num_fields); */
 	for(j=0;j<num_fields;j++)
 	{
 		type = DBFGetFieldInfo(hDBFHandle, j, name, &field_width, &field_precision); 
 
-//fprintf(stderr, "Field %d (%s) width/decimals: %d/%d\n", j, name, field_width, field_precision);
+/*fprintf(stderr, "Field %d (%s) width/decimals: %d/%d\n", j, name, field_width, field_precision); */
 		types[j] = type;
 		widths[j] = field_width;
 		precisions[j] = field_precision;
 
+#ifdef USE_ICONV
+		if ( encoding )
+		{
+			utf8str = utf8(encoding, name);
+			if ( ! utf8str ) exit(1);
+			strcpy(name, utf8str);
+			free(utf8str);
+		}
+#endif
+
+
 		/*
 		 * Make field names lowercase unless asked to
 		 * keep identifiers case.
 		 */
-		if ( ! quoteidentifiers ) {
-			for(z=0; z<strlen(name) ;z++)
-				name[z] = tolower(name[z]);
-		}
+		if ( ! quoteidentifiers ) LowerCase(name);
 
 		/*
 		 * Escape names starting with the
-		 * escape char (_)
+		 * escape char (_), those named 'gid'
+		 * or after pgsql reserved attribute names
 		 */
-		if( name[0]=='_' )
-		{
-			strcpy(name2+2, name);
-			name2[0] = '_';
-			name2[1] = '_';
-			strcpy(name, name2);
-		}
-
-		/*
-		 * Escape attributes named 'gid'
-		 * and pgsql reserved attribute names
-		 */
-		else if(
+		if( name[0]=='_' ||
 			! strcmp(name,"gid") ||
 			! strcmp(name, "tableoid") ||
 			! strcmp(name, "cmax") ||
@@ -1541,8 +1622,8 @@ GetFieldsSpec(void)
 			! strcmp(name, "cmin") ||
 			! strcmp(name, "primary") ||
 			! strcmp(name, "oid") ||
-			! strcmp(name, "ctid")
-		) {
+			! strcmp(name, "ctid") )
+		{
 			strcpy(name2+2, name);
 			name2[0] = '_';
 			name2[1] = '_';
@@ -1553,24 +1634,26 @@ GetFieldsSpec(void)
 		for(z=0; z < j ; z++){
 			if(strcmp(field_names[z],name)==0){
 				strcat(name,"__");
-				sprintf(name,"%s%i",name,j);
+				sprintf(name+strlen(name),"%i",j);
 				break;
 			}
 		}	
 
-
-		field_names[j] = malloc ( strlen(name)+3 );
+		field_names[j] = malloc (strlen(name)+1);
 		strcpy(field_names[j], name);
 
-		sprintf(col_names, "%s\"%s\",", col_names, name);
+		/*sprintf(col_names, "%s\"%s\",", col_names, name);*/
+		strcat(col_names, "\"");
+		strcat(col_names, name);
+		strcat(col_names, "\",");
 	}
-	sprintf(col_names, "%s\"%s\")", col_names, geom);
+	/*sprintf(col_names, "%s\"%s\")", col_names, geom);*/
+	strcat(col_names, geom);
+	strcat(col_names, ")");
 }
 
 #ifdef USE_ICONV
 
-#include <iconv.h>
-
 char *
 utf8 (const char *fromcode, char *inputbuf)
 {
@@ -1589,8 +1672,8 @@ utf8 (const char *fromcode, char *inputbuf)
 		return NULL;
 	}
 
-	outbytesleft = inbytesleft*3+1; // UTF8 string can be 3 times larger
-					// then local string
+	outbytesleft = inbytesleft*3+1; /* UTF8 string can be 3 times larger */
+					/* then local string */
 	outputbuf = (char *) malloc (outbytesleft);
 	if (!outputbuf)
 	{
@@ -1611,10 +1694,72 @@ utf8 (const char *fromcode, char *inputbuf)
 	return outputbuf;
 }
 
-#endif // defined USE_ICONV
+#endif /* defined USE_ICONV */
 
 /**********************************************************************
  * $Log: shp2pgsql.c,v $
+ * Revision 1.106  2006/01/16 10:42:58  strk
+ * Added support for Bool and Date DBF<=>PGIS mapping
+ *
+ * Revision 1.105  2006/01/09 16:40:16  strk
+ * ISO C90 comments, signedness mismatch fixes
+ *
+ * Revision 1.104  2005/11/01 09:25:47  strk
+ * Reworked NULL geometries handling code letting user specify policy (insert,skip,abort). Insert is the default.
+ *
+ * Revision 1.103  2005/10/24 15:54:22  strk
+ * fixed wrong assumption about maximum size of integer attributes (width is maximum size of text representation)
+ *
+ * Revision 1.102  2005/10/24 11:30:59  strk
+ *
+ * Fixed a bug in string attributes handling truncating values of maximum
+ * allowed length, curtesy of Lars Roessiger.
+ * Reworked integer attributes handling to be stricter in dbf->sql mapping
+ * and to allow for big int8 values in sql->dbf conversion
+ *
+ * Revision 1.101  2005/10/21 11:33:55  strk
+ * Applied patch by Lars Roessiger handling numerical values with a trailing decima
+ * l dot
+ *
+ * Revision 1.100  2005/10/13 13:40:20  strk
+ * Fixed return code from shp2pgsql
+ *
+ * Revision 1.99  2005/10/03 18:08:55  strk
+ * Stricter string attributes lenght handling. DBF header will be used
+ * to set varchar maxlenght, (var)char typmod will be used to set DBF header
+ * len.
+ *
+ * Revision 1.98  2005/10/03 07:45:58  strk
+ * Issued a warning when -W is specified and no UTF8 support has been compiled in.
+ *
+ * Revision 1.97  2005/09/30 08:59:29  strk
+ * Fixed release of stack memory occurring when shp2pgsql is compiled with USE_ICONV defined, an attribute value needs to be escaped and no -W is used
+ *
+ * Revision 1.96  2005/08/29 22:36:25  strk
+ * Removed premature object destruction in InsertLineString{WKT,} causing segfault
+ *
+ * Revision 1.95  2005/08/29 11:48:33  strk
+ * Fixed sprintf() calls to avoid overlapping memory,
+ * reworked not-null objects existance check to reduce startup costs.
+ *
+ * Revision 1.94  2005/07/27 02:47:14  strk
+ * Support for multibyte field names in loader
+ *
+ * Revision 1.93  2005/07/27 02:35:50  strk
+ * Minor cleanups in loader
+ *
+ * Revision 1.92  2005/07/27 02:07:01  strk
+ * Fixed handling of POINT types as WKT (-w) in loader
+ *
+ * Revision 1.91  2005/07/04 09:47:03  strk
+ * Added conservative iconv detection code
+ *
+ * Revision 1.90  2005/06/16 17:55:58  strk
+ * Added -I switch for GiST index creation in loader
+ *
+ * Revision 1.89  2005/04/21 09:08:34  strk
+ * Applied patch from Ron Mayer fixing a segfault in string escaper funx
+ *
  * Revision 1.88  2005/04/14 12:58:59  strk
  * Applied patch by Gino Lucrezi fixing bug in string escaping code.
  *
