@@ -25,13 +25,24 @@ expand_box2d(BOX2DFLOAT4 *box, double d)
 }
 
 
+/*
+ * This has been changed in PostGIS 1.1.2 to
+ * check exact equality of values (rather then using
+ * the FPeq macro taking into account coordinate drifts).
+ */
 char
 box2d_same(BOX2DFLOAT4 *box1, BOX2DFLOAT4 *box2)
 {
+	return(	(box1->xmax==box2->xmax) &&
+		(box1->xmin==box2->xmin) &&
+		(box1->ymax==box2->ymax) &&
+		(box1->ymin==box2->ymin));
+#if 0 // changed to exact equality
 	return(FPeq(box1->xmax, box2->xmax) &&
 				   FPeq(box1->xmin, box2->xmin) &&
 				   FPeq(box1->ymax, box2->ymax) &&
 				   FPeq(box1->ymin, box2->ymin));
+#endif // 0
 }
 
 BOX2DFLOAT4 *
