@@ -1,3 +1,15 @@
+/**********************************************************************
+ * $Id: lwgeom_functions_analytic.c,v 1.33 2006/01/29 13:54:38 strk Exp $
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.refractions.net
+ * Copyright 2001-2005 Refractions Research Inc.
+ *
+ * This is free software; you can redistribute and/or modify it under
+ * the terms of the GNU General Public Licence. See the COPYING file.
+ * 
+ **********************************************************************/
+
 #include "postgres.h"
 #include "fmgr.h"
 #include "liblwgeom.h"
@@ -95,13 +107,16 @@ elog(NOTICE, "DP_findsplit: segment too short, no split/no dist");
 POINTARRAY *
 DP_simplify2d(POINTARRAY *inpts, double epsilon)
 {
-	int stack[inpts->npoints];	/* recursion stack */
+	int *stack;			/* recursion stack */
 	int sp=-1;			/* recursion stack pointer */
 	int p1, split; 
 	double dist;
 	POINTARRAY *outpts;
 	int ptsize = pointArray_ptsize(inpts);
 
+	/* Allocate recursion stack */
+	stack = lwalloc(sizeof(int)*inpts->npoints);
+
 	p1 = 0;
 	stack[++sp] = inpts->npoints-1;
 
@@ -109,11 +124,11 @@ DP_simplify2d(POINTARRAY *inpts, double epsilon)
 	elog(NOTICE, "DP_simplify called input has %d pts and %d dims (ptsize: %d)", inpts->npoints, inpts->ndims, ptsize);
 #endif
 
-	// allocate space for output POINTARRAY
+	/* allocate space for output POINTARRAY */
 	outpts = palloc(sizeof(POINTARRAY));
 	outpts->dims = inpts->dims;
 	outpts->npoints=1;
-	outpts->serialized_pointlist = (char *)palloc(ptsize*inpts->npoints);
+	outpts->serialized_pointlist = palloc(ptsize*inpts->npoints);
 	memcpy(getPoint_internal(outpts, 0), getPoint_internal(inpts, 0),
 		ptsize);
 
@@ -155,7 +170,7 @@ DP_simplify2d(POINTARRAY *inpts, double epsilon)
 	 */
 	if ( outpts->npoints < inpts->npoints )
 	{
-		outpts->serialized_pointlist = (char *)repalloc(
+		outpts->serialized_pointlist = repalloc(
 			outpts->serialized_pointlist,
 			ptsize*outpts->npoints);
 		if ( outpts->serialized_pointlist == NULL ) {
@@ -163,6 +178,7 @@ DP_simplify2d(POINTARRAY *inpts, double epsilon)
 		}
 	}
 
+	lwfree(stack);
 	return outpts;
 }
 
@@ -184,7 +200,6 @@ simplify2d_lwline(const LWLINE *iline, double dist)
 	return oline;
 }
 
-// TODO
 LWPOLY *
 simplify2d_lwpoly(const LWPOLY *ipoly, double dist)
 {
@@ -344,7 +359,7 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 	LWPOINT *point;
 	POINTARRAY *ipa, *opa;
 	POINT4D pt;
-	char *srl;
+	uchar *srl;
 	int nsegs, i;
 	double length, slength, tlength;
 
@@ -371,7 +386,7 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 		else
 			getPoint4d_p(ipa, ipa->npoints-1, &pt);
 
-		opa = pointArray_construct((char *)&pt,
+		opa = pointArray_construct((uchar *)&pt,
 			TYPE_HASZ(line->type),
 			TYPE_HASM(line->type),
 			1);
@@ -386,13 +401,16 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 	length = lwgeom_pointarray_length2d(ipa);
 	tlength = 0;
 	for( i = 0; i < nsegs; i++ ) {
-		POINT2D p1, p2;
+		POINT4D p1, p2;
+		POINT4D *p1ptr=&p1, *p2ptr=&p2; /* don't break
+		                                 * strict-aliasing rules
+		                                 */
 
-		getPoint2d_p(ipa, i, &p1);
-		getPoint2d_p(ipa, i+1, &p2);
+		getPoint4d_p(ipa, i, &p1);
+		getPoint4d_p(ipa, i+1, &p2);
 
 		/* Find the relative length of this segment */
-		slength = distance2d_pt_pt(&p1, &p2)/length;
+		slength = distance2d_pt_pt((POINT2D*)p1ptr, (POINT2D*)p2ptr)/length;
 
 		/* If our target distance is before the total length we've seen
 		 * so far. create a new point some distance down the current
@@ -400,11 +418,8 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 		 */
 		if( distance < tlength + slength ) {
 			double dseg = (distance - tlength) / slength;
-			pt.x = (p1.x) + ((p2.x - p1.x) * dseg);
-			pt.y = (p1.y) + ((p2.y - p1.y) * dseg);
-			pt.z = 0;
-			pt.m = 0;
-			opa = pointArray_construct((char *)&pt,
+			interpolate_point4d(&p1, &p2, &pt, dseg);
+			opa = pointArray_construct((uchar *)&pt,
 				TYPE_HASZ(line->type),
 				TYPE_HASM(line->type),
 				1);
@@ -419,7 +434,7 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 	/* Return the last point on the line. This shouldn't happen, but
 	 * could if there's some floating point rounding errors. */
 	getPoint4d_p(ipa, ipa->npoints-1, &pt);
-	opa = pointArray_construct((char *)&pt,
+	opa = pointArray_construct((uchar *)&pt,
 		TYPE_HASZ(line->type),
 		TYPE_HASM(line->type),
 		1);
@@ -491,10 +506,15 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 typedef struct gridspec_t {
    double ipx;
    double ipy;
+   double ipz;
+   double ipm;
    double xsize;
    double ysize;
+   double zsize;
+   double msize;
 } gridspec;
 
+
 /* Forward declarations */
 LWGEOM *lwgeom_grid(LWGEOM *lwgeom, gridspec *grid);
 LWCOLLECTION *lwcollection_grid(LWCOLLECTION *coll, gridspec *grid);
@@ -503,6 +523,33 @@ LWPOLY * lwpoly_grid(LWPOLY *poly, gridspec *grid);
 LWLINE *lwline_grid(LWLINE *line, gridspec *grid);
 POINTARRAY *ptarray_grid(POINTARRAY *pa, gridspec *grid);
 Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS);
+Datum LWGEOM_snaptogrid_pointoff(PG_FUNCTION_ARGS);
+static int grid_isNull(const gridspec *grid);
+#if VERBOSE
+static void grid_print(const gridspec *grid, lwreporter printer);
+#endif
+
+/* A NULL grid is a grid in which size in all dimensions is 0 */
+static int
+grid_isNull(const gridspec *grid)
+{
+	if ( grid->xsize==0 &&
+		grid->ysize==0 &&
+		grid->zsize==0 &&
+		grid->msize==0 ) return 1;
+	else return 0;
+}
+
+#if VERBOSE
+/* Print grid using given reporter */
+static void
+grid_print(const gridspec *grid, lwreporter printer)
+{
+	printer("GRID(%g %g %g %g, %g %g %g %g)",
+		grid->ipx, grid->ipy, grid->ipz, grid->ipm,
+		grid->xsize, grid->ysize, grid->zsize, grid->msize);
+}
+#endif
 
 /*
  * Stick an array of points to the given gridspec.
@@ -515,106 +562,44 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS);
 POINTARRAY *
 ptarray_grid(POINTARRAY *pa, gridspec *grid)
 {
-	POINT2D pbuf; 
-	int numoutpoints=0;
-	int pn; /* point number */
-	char *opts;
+	POINT4D pbuf; 
+	int ipn, opn; /* point numbers (input/output) */
+	DYNPTARRAY *dpa;
 	POINTARRAY *opa;
-	size_t ptsize;
 
 #if VERBOSE
 	elog(NOTICE, "ptarray_grid called on %p", pa);
 #endif
 
-	ptsize = sizeof(POINT2D);
+	dpa=dynptarray_create(pa->npoints, pa->dims);
 
-#if VERBOSE
-	elog(NOTICE, "ptarray_grid: ptsize: %d", ptsize);
-#endif
-
-	opts = (char *)palloc(ptsize * pa->npoints);
-	if ( opts == NULL )
+	for (ipn=0, opn=0; ipn<pa->npoints; ++ipn)
 	{
-		elog(ERROR, "Out of virtual memory");
-		return NULL;
-	}
 
-	for (pn=0; pn<pa->npoints; pn++)
-	{
-		POINT2D *lastpoint = NULL;
-		POINT2D *lastpoint2 = NULL;
-
-		getPoint2d_p(pa, pn, &pbuf);
+		getPoint4d_p(pa, ipn, &pbuf);
 
 		if ( grid->xsize )
 			pbuf.x = rint((pbuf.x - grid->ipx)/grid->xsize) *
 				grid->xsize + grid->ipx;
+
 		if ( grid->ysize )
 			pbuf.y = rint((pbuf.y - grid->ipy)/grid->ysize) *
 				grid->ysize + grid->ipy;
 
-		/* Points have been already drawn */
-		if ( numoutpoints )
-		{
-			lastpoint = (POINT2D *)(opts+((numoutpoints-1)*ptsize));
+		if ( TYPE_HASZ(pa->dims) && grid->zsize )
+			pbuf.z = rint((pbuf.z - grid->ipz)/grid->zsize) *
+				grid->zsize + grid->ipz;
 
-			/*
-			 * Skip point if falling on the same cell then
-			 * the previous one
-			 */
-			if ( SAMEPOINT(&pbuf, lastpoint) ) continue;
+		if ( TYPE_HASM(pa->dims) && grid->msize )
+			pbuf.m = rint((pbuf.m - grid->ipm)/grid->msize) *
+				grid->msize + grid->ipm;
 
-			/*
-			 * Skip this and previous point if they make a
-			 * back_and_forw line.
-			 * WARNING! this migth be a valid reduction only for
-			 * polygons, since a linestring might want to
-			 * preserve that!
-			 */
-			if ( numoutpoints )
-			{
-				lastpoint2 = (POINT2D *)(opts+((numoutpoints-2)*ptsize));
-				if (  SAMEPOINT(&pbuf, lastpoint2) )
-				{
-					numoutpoints--;
-					continue;
-				}
-			}
-		}
-
-		memcpy(opts+(numoutpoints*ptsize), &pbuf, ptsize);
-		numoutpoints++;
+		dynptarray_addPoint4d(dpa, &pbuf, 0);
 
 	}
 
-	if ( numoutpoints )
-	{
-		/* Srhink allocated memory for outpoints if needed */
-		if ( numoutpoints < pa->npoints )
-		{
-			opts = (char *)repalloc(opts, ptsize*numoutpoints);
-			if ( opts == NULL )
-			{
-				elog(ERROR, "Out of virtual memory");
-				return NULL;
-			}
-		}
-	}
-
-	/*
-	 * This should never happen since no reduction occurs if not
-	 * based on other drawn points
-	 */
-	else
-	{
-		elog(NOTICE, "No points drawn out of %d input points, error?",
-			pa->npoints);
-		pfree(opts);
-		opts = NULL;
-		return NULL;
-	}
-
-	opa = pointArray_construct(opts, 0, 0, numoutpoints);
+	opa = dpa->pa;
+	lwfree(dpa);
 
 	return opa;
 }
@@ -630,7 +615,7 @@ lwline_grid(LWLINE *line, gridspec *grid)
 	/* Skip line3d with less then 2 points */
 	if ( opa->npoints < 2 ) return NULL;
 
-	// TODO: grid bounding box...
+	/* TODO: grid bounding box... */
 	oline = lwline_construct(line->SRID, NULL, opa);
 
 	return oline;
@@ -730,18 +715,19 @@ elog(NOTICE, "grid_polygon3d: simplified polygon with %d rings", nrings);
 LWPOINT *
 lwpoint_grid(LWPOINT *point, gridspec *grid)
 {
-	POINT2D p;
-	double x, y;
-	getPoint2d_p(point->point, 0, &p);
-	x = rint((p.x - grid->ipx)/grid->xsize) *
-		grid->xsize + grid->ipx;
-	y = rint((p.y - grid->ipy)/grid->ysize) *
-		grid->ysize + grid->ipy;
+	LWPOINT *opoint;
+	POINTARRAY *opa;
+
+	opa = ptarray_grid(point->point, grid);
+
+	/* TODO: grid bounding box ? */
+	opoint = lwpoint_construct(point->SRID, NULL, opa);
 
 #if VERBOSE
 	elog(NOTICE, "lwpoint_grid called");
 #endif
-	return make_lwpoint2d(point->SRID, x, y);
+
+	return opoint;
 }
 
 LWCOLLECTION *
@@ -797,7 +783,7 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS)
 	PG_LWGEOM *out_geom = NULL;
 	LWGEOM *out_lwgeom;
 	gridspec grid;
-	BOX2DFLOAT4 *box;
+	//BOX3D box3d;
 
 	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
 	datum = PG_GETARG_DATUM(0);
@@ -815,8 +801,14 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS)
 	if ( PG_ARGISNULL(4) ) PG_RETURN_NULL();
 	grid.ysize = PG_GETARG_FLOAT8(4);
 
-	/* 0-sided grid == grid */
-	if ( grid.xsize == 0 && grid.ysize == 0 ) PG_RETURN_POINTER(in_geom);
+	/* Do not support gridding Z and M values for now */
+	grid.ipz=grid.ipm=grid.zsize=grid.msize=0;
+
+	/* Return input geometry if grid is null */
+	if ( grid_isNull(&grid) )
+	{
+		PG_RETURN_POINTER(in_geom);
+	}
 
 	in_lwgeom = lwgeom_deserialize(SERIALIZED_FORM(in_geom));
 
@@ -827,20 +819,36 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS)
    	out_lwgeom = lwgeom_grid(in_lwgeom, &grid);
 	if ( out_lwgeom == NULL ) PG_RETURN_NULL();
 
-	/* COMPUTE_BBOX WHEN_SIMPLE */
+	/* COMPUTE_BBOX TAINTING */
+	if ( in_lwgeom->bbox ) lwgeom_addBBOX(out_lwgeom);
+
+#if 0
+	/*
+	 * COMPUTE_BBOX WHEN SIMPLE
+	 *
+	 * WARNING: this is not SIMPLE, as snapping
+	 * an existing bbox to a grid does not
+	 * give the same result as computing a
+	 * new bounding box on the snapped coordinates.
+	 *
+	 * This bug has been fixed in postgis-1.1.2
+	 */
 	if ( in_lwgeom->bbox )
 	{
-		box = palloc(sizeof(BOX2DFLOAT4));
-		box->xmin = rint((in_lwgeom->bbox->xmin - grid.ipx)/grid.xsize)
+		box2df_to_box3d_p(in_lwgeom->bbox, &box3d);
+
+		box3d.xmin = rint((box3d.xmin - grid.ipx)/grid.xsize)
 			* grid.xsize + grid.ipx;
-		box->xmax = rint((in_lwgeom->bbox->xmax - grid.ipx)/grid.xsize)
+		box3d.xmax = rint((box3d.xmax - grid.ipx)/grid.xsize)
 			* grid.xsize + grid.ipx;
-		box->ymin = rint((in_lwgeom->bbox->ymin - grid.ipy)/grid.ysize)
+		box3d.ymin = rint((box3d.ymin - grid.ipy)/grid.ysize)
 			* grid.ysize + grid.ipy;
-		box->ymax = rint((in_lwgeom->bbox->ymax - grid.ipy)/grid.ysize)
+		box3d.ymax = rint((box3d.ymax - grid.ipy)/grid.ysize)
 			* grid.ysize + grid.ipy;
-		out_lwgeom->bbox = box;
+
+		out_lwgeom->bbox = box3d_to_box2df(&box3d);
 	}
+#endif // 0
 
 #if VERBOSE
 	elog(NOTICE, "SnapToGrid made a %s", lwgeom_typename(TYPE_GETTYPE(out_lwgeom->type)));
@@ -850,6 +858,200 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS)
 
 	PG_RETURN_POINTER(out_geom);
 }
+
+PG_FUNCTION_INFO_V1(LWGEOM_snaptogrid_pointoff);
+Datum LWGEOM_snaptogrid_pointoff(PG_FUNCTION_ARGS)
+{
+	Datum datum;
+	PG_LWGEOM *in_geom, *in_point;
+	LWGEOM *in_lwgeom;
+	LWPOINT *in_lwpoint;
+	PG_LWGEOM *out_geom = NULL;
+	LWGEOM *out_lwgeom;
+	gridspec grid;
+	//BOX3D box3d;
+	POINT4D offsetpoint;
+
+	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
+	datum = PG_GETARG_DATUM(0);
+	in_geom = (PG_LWGEOM *)PG_DETOAST_DATUM(datum);
+
+	if ( PG_ARGISNULL(1) ) PG_RETURN_NULL();
+	datum = PG_GETARG_DATUM(1);
+	in_point = (PG_LWGEOM *)PG_DETOAST_DATUM(datum);
+	in_lwpoint = lwpoint_deserialize(SERIALIZED_FORM(in_point));
+	if ( in_lwpoint == NULL )
+	{
+		lwerror("Offset geometry must be a point");
+	}
+
+	if ( PG_ARGISNULL(2) ) PG_RETURN_NULL();
+	grid.xsize = PG_GETARG_FLOAT8(2);
+
+	if ( PG_ARGISNULL(3) ) PG_RETURN_NULL();
+	grid.ysize = PG_GETARG_FLOAT8(3);
+
+	if ( PG_ARGISNULL(4) ) PG_RETURN_NULL();
+	grid.zsize = PG_GETARG_FLOAT8(4);
+
+	if ( PG_ARGISNULL(5) ) PG_RETURN_NULL();
+	grid.msize = PG_GETARG_FLOAT8(5);
+
+	/* Take offsets from point geometry */
+	getPoint4d_p(in_lwpoint->point, 0, &offsetpoint);
+	grid.ipx = offsetpoint.x;
+	grid.ipy = offsetpoint.y;
+	if (TYPE_HASZ(in_lwpoint->type) ) grid.ipz = offsetpoint.z;
+	else grid.ipz=0;
+	if (TYPE_HASM(in_lwpoint->type) ) grid.ipm = offsetpoint.m;
+	else grid.ipm=0;
+
+#if VERBOSE
+	grid_print(&grid, lwnotice);
+#endif
+
+	/* Return input geometry if grid is null */
+	if ( grid_isNull(&grid) )
+	{
+		PG_RETURN_POINTER(in_geom);
+	}
+
+	in_lwgeom = lwgeom_deserialize(SERIALIZED_FORM(in_geom));
+
+#if VERBOSE
+	elog(NOTICE, "SnapToGrid got a %s", lwgeom_typename(TYPE_GETTYPE(in_lwgeom->type)));
+#endif
+
+   	out_lwgeom = lwgeom_grid(in_lwgeom, &grid);
+	if ( out_lwgeom == NULL ) PG_RETURN_NULL();
+
+	/* COMPUTE_BBOX TAINTING */
+	if ( in_lwgeom->bbox ) lwgeom_addBBOX(out_lwgeom);
+
+#if 0
+	/*
+	 * COMPUTE_BBOX WHEN SIMPLE
+	 *
+	 * WARNING: this is not SIMPLE, as snapping
+	 * an existing bbox to a grid does not
+	 * give the same result as computing a
+	 * new bounding box on the snapped coordinates.
+	 *
+	 * This bug has been fixed in postgis-1.1.2
+	 */
+	if ( in_lwgeom->bbox )
+	{
+		box2df_to_box3d_p(in_lwgeom->bbox, &box3d);
+
+		box3d.xmin = rint((box3d.xmin - grid.ipx)/grid.xsize)
+			* grid.xsize + grid.ipx;
+		box3d.xmax = rint((box3d.xmax - grid.ipx)/grid.xsize)
+			* grid.xsize + grid.ipx;
+		box3d.ymin = rint((box3d.ymin - grid.ipy)/grid.ysize)
+			* grid.ysize + grid.ipy;
+		box3d.ymax = rint((box3d.ymax - grid.ipy)/grid.ysize)
+			* grid.ysize + grid.ipy;
+
+		out_lwgeom->bbox = box3d_to_box2df(&box3d);
+	}
+#endif // 0
+
+#if VERBOSE
+	elog(NOTICE, "SnapToGrid made a %s", lwgeom_typename(TYPE_GETTYPE(out_lwgeom->type)));
+#endif
+
+	out_geom = pglwgeom_serialize(out_lwgeom);
+
+	PG_RETURN_POINTER(out_geom);
+}
+
 /***********************************************************************
  * --strk@keybit.net
  ***********************************************************************/
+
+Datum LWGEOM_line_substring(PG_FUNCTION_ARGS);
+
+PG_FUNCTION_INFO_V1(LWGEOM_line_substring);
+Datum LWGEOM_line_substring(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	double from = PG_GETARG_FLOAT8(1);
+	double to = PG_GETARG_FLOAT8(2);
+	LWLINE *iline;
+	LWGEOM *olwgeom;
+	POINTARRAY *ipa, *opa;
+	PG_LWGEOM *ret;
+
+	if( from < 0 || from > 1 ) {
+		elog(ERROR,"line_interpolate_point: 2nd arg isnt within [0,1]");
+		PG_RETURN_NULL();
+	}
+
+	if( to < 0 || to > 1 ) {
+		elog(ERROR,"line_interpolate_point: 3rd arg isnt within [0,1]");
+		PG_RETURN_NULL();
+	}
+
+	if ( from > to ) {
+		elog(ERROR, "2nd arg must be smaller then 3rd arg");
+		PG_RETURN_NULL();
+	}
+
+	if( lwgeom_getType(geom->type) != LINETYPE ) {
+		elog(ERROR,"line_interpolate_point: 1st arg isnt a line");
+		PG_RETURN_NULL();
+	}
+
+	iline = lwline_deserialize(SERIALIZED_FORM(geom));
+	ipa = iline->points;
+
+	opa = ptarray_substring(ipa, from, to);
+
+	if ( opa->npoints == 1 ) // Point returned
+		olwgeom = (LWGEOM *)lwpoint_construct(iline->SRID, NULL, opa);
+	else
+		olwgeom = (LWGEOM *)lwline_construct(iline->SRID, NULL, opa);
+
+	ret = pglwgeom_serialize(olwgeom);
+	PG_FREE_IF_COPY(geom, 0);
+	lwgeom_release(olwgeom);
+	PG_RETURN_POINTER(ret);
+}
+
+Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS);
+
+PG_FUNCTION_INFO_V1(LWGEOM_line_locate_point);
+Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	LWLINE *lwline;
+	LWPOINT *lwpoint;
+	POINTARRAY *pa;
+	POINT2D p;
+	double ret;
+
+	if( lwgeom_getType(geom1->type) != LINETYPE ) {
+		elog(ERROR,"line_locate_point: 1st arg isnt a line");
+		PG_RETURN_NULL();
+	}
+	if( lwgeom_getType(geom2->type) != POINTTYPE ) {
+		elog(ERROR,"line_locate_point: 2st arg isnt a point");
+		PG_RETURN_NULL();
+	}
+	if ( pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2) )
+	{
+		elog(ERROR, "Operation on two geometries with different SRIDs");
+		PG_RETURN_NULL();
+	}
+
+	lwline = lwline_deserialize(SERIALIZED_FORM(geom1));
+	lwpoint = lwpoint_deserialize(SERIALIZED_FORM(geom2));
+
+	pa = lwline->points;
+	lwpoint_getPoint2d_p(lwpoint, &p);
+
+	ret = ptarray_locate_point(pa, &p);
+
+	PG_RETURN_FLOAT8(ret);
+}
