@@ -5,6 +5,7 @@
 #include <string>
 #include <iostream>
 #include <fstream>
+#include <typeinfo>
 
 #include "postgis_geos_version.h"
 #include "geos/geom.h"
@@ -12,6 +13,7 @@
 #if GEOS_FIRST_INTERFACE <= 3 && GEOS_LAST_INTERFACE >= 3
 #include "geos/opValid.h"
 #include "geos/opPolygonize.h"
+#include "geos/opLinemerge.h"
 #endif // GEOS_FIRST_INTERFACE <= 3 && GEOS_LAST_INTERFACE >= 3
 
 //#define DEBUG_POSTGIS2GEOS 1
@@ -109,6 +111,7 @@ extern "C" char GEOSrelateWithin(Geometry *g1, Geometry*g2);
 extern "C" char GEOSrelateContains(Geometry *g1, Geometry*g2);
 extern "C" char GEOSrelateOverlaps(Geometry *g1, Geometry*g2);
 extern "C" Geometry *GEOSpolygonize(Geometry **geoms, unsigned int ngeoms);
+extern "C" Geometry *GEOSLineMerge(Geometry *geoms);
 extern "C" char *GEOSversion();
 extern "C" char *GEOSjtsport();
 
@@ -154,9 +157,17 @@ extern "C" char GEOSequals(Geometry *g1, Geometry*g2);
 extern "C" char GEOSisRing(Geometry *g1);
 extern "C" Geometry *GEOSpointonSurface(Geometry *g1);
 extern "C" Geometry *GEOSGetCentroid(Geometry *g1, int *failure);
-extern "C" void NOTICE_MESSAGE(char *msg);
+extern "C" void NOTICE_MESSAGE(const char *msg);
 extern "C" bool GEOSHasZ(Geometry *g1);
 
+/*
+ * Returned Polygon will takes ownership of all args
+ * Taken from GEOS CAPI
+ */
+extern "C" Geometry *GEOSCreatePolygon(Geometry *shell, Geometry **holes, 
+	unsigned int nholes);
+extern "C" Geometry *GEOSCloneGeometry(const Geometry *g);
+
 //###########################################################
 #if GEOS_LAST_INTERFACE < 2
 # define CoordinateSequence CoordinateList
@@ -208,7 +219,7 @@ Geometry *PostGIS2GEOS_box3d(BOX3D *box, int SRID)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete cl;
 		return NULL;
 	}
@@ -258,7 +269,7 @@ Geometry *PostGIS2GEOS_point(const LWPOINT *lwpoint)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -343,7 +354,7 @@ PostGIS2GEOS_linestring(const LWLINE *lwline)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -496,7 +507,7 @@ Geometry *PostGIS2GEOS_polygon(const LWPOLY *lwpoly)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -533,7 +544,7 @@ Geometry *PostGIS2GEOS_multipoint(LWPOINT **geoms, uint32 ngeoms, int SRID, bool
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -570,7 +581,7 @@ Geometry *PostGIS2GEOS_multilinestring(LWLINE **geoms, uint32 ngeoms, int SRID,
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -607,7 +618,7 @@ Geometry *PostGIS2GEOS_multipolygon(LWPOLY **polygons, uint32 npolys, int SRID,
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -667,7 +678,7 @@ Geometry *PostGIS2GEOS_collection(int type, Geometry **geoms, int ngeoms, int SR
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -693,7 +704,7 @@ char GEOSrelateDisjoint(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -712,7 +723,7 @@ char GEOSrelateTouches(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -732,7 +743,7 @@ char GEOSrelateIntersects(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -752,7 +763,7 @@ char GEOSrelateCrosses(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -772,7 +783,7 @@ char GEOSrelateWithin(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -796,7 +807,7 @@ char GEOSrelateContains(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -816,7 +827,7 @@ char GEOSrelateOverlaps(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -842,7 +853,7 @@ char GEOSrelatePattern(Geometry *g1, Geometry*g2,char *pat)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -868,12 +879,13 @@ char *GEOSrelate(Geometry *g1, Geometry*g2)
 		s= im->toString();
 		result = (char*) malloc( s.length() + 1);
 		strcpy(result, s.c_str() );
+		delete im;
 
 		return result;
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -905,7 +917,7 @@ char GEOSisvalid(Geometry *g1)
 			TopologyValidationError *err = ivo.getValidationError();
 			if ( err ) {
 				string errmsg = err->getMessage();
-				NOTICE_MESSAGE((char *)errmsg.c_str());
+				NOTICE_MESSAGE(errmsg.c_str());
 			}
 		}
 #else // GEOS_INTERFACE 3 not supported
@@ -915,7 +927,7 @@ char GEOSisvalid(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -941,7 +953,7 @@ char GEOSequals(Geometry *g1, Geometry*g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -968,7 +980,7 @@ char *GEOSasText(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -987,7 +999,7 @@ char GEOSisEmpty(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -1006,7 +1018,7 @@ char GEOSisSimple(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -1025,7 +1037,7 @@ char GEOSisRing(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 2;
 	}
@@ -1053,7 +1065,7 @@ char *GEOSGeometryType(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1095,7 +1107,7 @@ int GEOSGeometryTypeId(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return -1;
 	}
@@ -1122,7 +1134,7 @@ Geometry *GEOSIntersection(Geometry *g1,Geometry *g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1143,7 +1155,7 @@ GEOSBuffer(Geometry *g1, double width, int quadrantsegments)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1163,7 +1175,7 @@ Geometry *GEOSConvexHull(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1183,7 +1195,7 @@ Geometry *GEOSDifference(Geometry *g1,Geometry *g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1203,7 +1215,7 @@ Geometry *GEOSBoundary(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1223,7 +1235,7 @@ Geometry *GEOSSymDifference(Geometry *g1,Geometry *g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1243,7 +1255,7 @@ Geometry *GEOSUnion(Geometry *g1,Geometry *g2)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1263,7 +1275,7 @@ Geometry *GEOSpointonSurface(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1284,14 +1296,15 @@ Geometry *GEOSpointonSurface(Geometry *g1)
 
 
 //BUG:: this leaks memory, but delete kills the PrecisionModel for ALL the geometries
-void GEOSdeleteGeometry(Geometry *a)
+void
+GEOSdeleteGeometry(Geometry *a)
 {
 	try{
 		delete a;
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		//return NULL;
 	}
@@ -1315,7 +1328,7 @@ void GEOSdeleteChar(char *a)
 	}
 	catch (GEOSException *ge) // ???
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		//return NULL;
 	}
@@ -1347,7 +1360,7 @@ POINT3D  *GEOSGetCoordinate(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1390,7 +1403,7 @@ POINT3D  *GEOSGetCoordinates(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1451,7 +1464,7 @@ POINT3D  *GEOSGetCoordinates_Polygon(Polygon *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1473,7 +1486,7 @@ int      GEOSGetNumCoordinate(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 0;
 	}
@@ -1492,7 +1505,7 @@ int      GEOSGetNumInteriorRings(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 0;
 	}
@@ -1513,7 +1526,7 @@ int      GEOSGetNumGeometries(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 0;
 	}
@@ -1534,7 +1547,7 @@ const Geometry *GEOSGetGeometryN(Geometry *g1, int n)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1555,7 +1568,7 @@ const Geometry *GEOSGetExteriorRing(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 0;
 	}
@@ -1575,7 +1588,7 @@ const Geometry *GEOSGetInteriorRingN(Geometry *g1,int n)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1596,7 +1609,7 @@ Geometry *GEOSGetCentroid(Geometry *g, int *failure)
 	catch (GEOSException *ge)
 	{
 		*failure = 1;
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1643,11 +1656,50 @@ Geometry *GEOSpolygonize(Geometry **g, unsigned int ngeoms)
 
 		geoms = new vector<Geometry *>(polys->size());
 		for (i=0; i<polys->size(); i++) (*geoms)[i] = (*polys)[i];
+		delete polys;
 		out = geomFactory->createGeometryCollection(geoms);
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
+		delete ge;
+		return NULL;
+	}
+	catch(...)
+	{
+		return NULL;
+	}
+
+	return out;
+}
+
+Geometry *
+GEOSLineMerge(Geometry *g)
+{
+	unsigned int i;
+	Geometry *out = NULL;
+
+	try{
+		// LineMerge
+		LineMerger lmrgr;
+
+		lmrgr.add(g);
+
+		vector<LineString *>*lines = lmrgr.getMergedLineStrings();
+
+#if DEBUG
+	NOTICE_MESSAGE("output lines got");
+#endif
+
+		vector<Geometry *>*geoms = new vector<Geometry *>(lines->size());
+		for (i=0; i<lines->size(); i++) (*geoms)[i] = (*lines)[i];
+		delete lines;
+		out = geomFactory->buildGeometry(geoms);
+		//out = geomFactory->createGeometryCollection(geoms);
+	}
+	catch (GEOSException *ge)
+	{
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return NULL;
 	}
@@ -1664,6 +1716,11 @@ Geometry *GEOSpolygonize(Geometry **g, unsigned int ngeoms)
 	NOTICE_MESSAGE("GEOS library does not support required interface 3");
 	return NULL;
 }
+Geometry *GEOSLineMerge(Geometry *g)
+{
+	NOTICE_MESSAGE("GEOS library does not support required interface 3");
+	return NULL;
+}
 #endif // ! (GEOS_FIRST_INTERFACE <= 3 && GEOS_LAST_INTERFACE >= 3)
 
 
@@ -1674,7 +1731,7 @@ int      GEOSGetSRID(Geometry *g1)
 	}
 	catch (GEOSException *ge)
 	{
-		NOTICE_MESSAGE((char *)ge->toString().c_str());
+		NOTICE_MESSAGE(ge->toString().c_str());
 		delete ge;
 		return 0;
 	}
@@ -1722,8 +1779,81 @@ bool
 GEOSHasZ(Geometry *g)
 {
 	//char msg[256];
+	if ( g->isEmpty() ) return false;
 	double az = g->getCoordinate()->z;
 	//sprintf(msg, "ZCoord: %g", az);
 	//NOTICE_MESSAGE(msg);
 	return (finite(az) && az != DoubleNotANumber);
 }
+
+
+/*
+ * Returned Polygon will takes ownership of all args
+ * Taken from GEOS CAPI
+ */
+Geometry *
+GEOSCreatePolygon(Geometry *shell, Geometry **holes, 
+	unsigned int nholes)
+{
+	try
+	{
+		vector<Geometry *> *vholes=new vector<Geometry *>(holes, holes+nholes);
+		LinearRing *nshell = dynamic_cast<LinearRing *>(shell);
+		if ( ! nshell )
+		{
+			NOTICE_MESSAGE("Shell is not LinearRing");
+			return NULL;
+		}
+		return geomFactory->createPolygon(nshell, vholes);
+	}
+	catch (GEOSException *ge)
+	{
+		NOTICE_MESSAGE(ge->toString().c_str());
+		delete ge;
+		return NULL;
+	}
+	catch (std::exception &e)
+	{
+		NOTICE_MESSAGE(e.what());
+		return NULL;
+	}
+
+	catch (...)
+	{
+		NOTICE_MESSAGE("Unknown exception thrown");
+		return NULL;
+	}
+}
+
+Geometry *
+GEOSCloneGeometry(const Geometry *g)
+{
+	try { 
+		/*
+		 * GEOS up to 2.2.1 did not properly cloned
+		 * LinearRing objects, so we handle it here
+		 */
+		if ( typeid(*g) == typeid(LinearRing) )
+		{
+			return new LinearRing((LinearRing&)*g);
+		}
+		return g->clone();
+	}
+	catch (GEOSException *ge)
+	{
+		NOTICE_MESSAGE(ge->toString().c_str());
+		delete ge;
+		return NULL;
+	}
+	catch (std::exception &e)
+	{
+		NOTICE_MESSAGE(e.what());
+		return NULL;
+	}
+
+	catch (...)
+	{
+		NOTICE_MESSAGE("Unknown exception thrown");
+		return NULL;
+	}
+}
