@@ -7,6 +7,9 @@
 #include "postgres.h"
 #include "access/gist.h"
 #include "access/itup.h"
+#if USE_VERSION > 80
+#include "access/skey.h"
+#endif
 #include "fmgr.h"
 #include "utils/elog.h"
 
@@ -14,17 +17,19 @@
 #include "lwgeom_pg.h"
 #include "stringBuffer.h"
 
-// implementation GiST support and basic LWGEOM operations (like &&)
+/*
+ * implementation GiST support and basic LWGEOM operations (like &&)
+ */
 
 
-//#define PGIS_DEBUG
-//#define PGIS_DEBUG_CALLS
-//#define PGIS_DEBUG_GIST
-//#define PGIS_DEBUG_GIST2
-//#define PGIS_DEBUG_GIST3
-//#define PGIS_DEBUG_GIST4
-//#define PGIS_DEBUG_GIST5
-//#define PGIS_DEBUG_GIST6
+/* #define PGIS_DEBUG */
+/* #define PGIS_DEBUG_CALLS */
+/* #define PGIS_DEBUG_GIST */
+/* #define PGIS_DEBUG_GIST2 */
+/* #define PGIS_DEBUG_GIST3 */
+/* #define PGIS_DEBUG_GIST4 */
+/* #define PGIS_DEBUG_GIST5 */
+/* #define PGIS_DEBUG_GIST6 */
 
 Datum LWGEOM_overlap(PG_FUNCTION_ARGS);
 Datum LWGEOM_overleft(PG_FUNCTION_ARGS);
@@ -49,17 +54,17 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS);
 
 static float size_box2d(Datum box);
 
-static bool lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key,BOX2DFLOAT4 *query,StrategyNumber strategy);
+static bool lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key, BOX2DFLOAT4 *query, StrategyNumber strategy);
 static bool lwgeom_rtree_leaf_consistent(BOX2DFLOAT4 *key,BOX2DFLOAT4 *query,	StrategyNumber strategy);
 
 
 
-// for debugging
-int counter_leaf = 0;
-int counter_intern = 0;
+/* for debugging */
+static int counter_leaf = 0;
+static int counter_intern = 0;
 
 
-// GiST strategies (modified from src/include/access/rtree.h)
+/* GiST strategies (modified from src/include/access/rtree.h) */
 #define RTLeftStrategyNumber			1
 #define RTOverLeftStrategyNumber		2
 #define RTOverlapStrategyNumber			3
@@ -74,34 +79,44 @@ int counter_intern = 0;
 #define RTOverAboveStrategyNumber		12
 
 
-// all the lwgeom_<same,overlpa,overleft,left,right,overright,overbelow,below,above,overabove,contained,contain>
-//  work the same.
-//  1. get lwgeom1
-//  2. get lwgeom2
-//  3. get box2d for lwgeom1
-//  4. get box2d for lwgeom2
-//  7. call the appropriate BOX2DFLOAT4 function
-//  8. return result;
+/*
+ * all the lwgeom_<same,overlpa,overleft,left,right,overright,overbelow,below,above,overabove,contained,contain>
+ *  work the same.
+ *  1. get lwgeom1
+ *  2. get lwgeom2
+ *  3. get box2d for lwgeom1
+ *  4. get box2d for lwgeom2
+ *  7. call the appropriate BOX2DFLOAT4 function
+ *  8. return result;
+ */
 
 PG_FUNCTION_INFO_V1(LWGEOM_overlap);
 Datum LWGEOM_overlap(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
+#ifdef PGIS_DEBUG_CALLS
+	elog(NOTICE,"GIST: LWGEOM_overlap --entry");
+#endif
+
+	if ( pglwgeom_getSRID(lwgeom1) != pglwgeom_getSRID(lwgeom2) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+        	PG_FREE_IF_COPY(lwgeom2, 1);
 		elog(ERROR, "Operation on two geometries with different SRIDs");
 		PG_RETURN_NULL();
 	}
 
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
-		// One or both are empty geoms
+		PG_FREE_IF_COPY(lwgeom1, 0);
+        	PG_FREE_IF_COPY(lwgeom2, 1);
+		/* One or both are empty geoms */
         	PG_RETURN_BOOL(FALSE);
 	}
 
@@ -131,8 +146,8 @@ Datum LWGEOM_overlap(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_overleft);
 Datum LWGEOM_overleft(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -141,14 +156,18 @@ Datum LWGEOM_overleft(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_overleft --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
+	if ( pglwgeom_getSRID(lwgeom1) != pglwgeom_getSRID(lwgeom2) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		elog(ERROR, "Operation on two geometries with different SRIDs");
 		PG_RETURN_NULL();
 	}
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -165,8 +184,8 @@ Datum LWGEOM_overleft(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_left);
 Datum LWGEOM_left(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -175,14 +194,12 @@ Datum LWGEOM_left(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_left --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -199,8 +216,8 @@ Datum LWGEOM_left(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_right);
 Datum LWGEOM_right(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -209,14 +226,12 @@ Datum LWGEOM_right(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_right --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -233,8 +248,8 @@ Datum LWGEOM_right(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_overright);
 Datum LWGEOM_overright(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -243,14 +258,12 @@ Datum LWGEOM_overright(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_overright --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -267,8 +280,8 @@ Datum LWGEOM_overright(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_overbelow);
 Datum LWGEOM_overbelow(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -277,14 +290,12 @@ Datum LWGEOM_overbelow(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_overbelow --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -301,8 +312,8 @@ Datum LWGEOM_overbelow(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_below);
 Datum LWGEOM_below(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -311,14 +322,12 @@ Datum LWGEOM_below(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_below --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -335,8 +344,8 @@ Datum LWGEOM_below(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_above);
 Datum LWGEOM_above(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -345,14 +354,12 @@ Datum LWGEOM_above(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_above --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -369,8 +376,8 @@ Datum LWGEOM_above(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_overabove);
 Datum LWGEOM_overabove(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -379,14 +386,12 @@ Datum LWGEOM_overabove(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_overabove --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -403,8 +408,8 @@ Datum LWGEOM_overabove(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_contained);
 Datum LWGEOM_contained(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -413,14 +418,12 @@ Datum LWGEOM_contained(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_contained --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -437,8 +440,8 @@ Datum LWGEOM_contained(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_contain);
 Datum LWGEOM_contain(PG_FUNCTION_ARGS)
 {
-	char *lwgeom1 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	char *lwgeom2 = (char *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 	bool result;
 	BOX2DFLOAT4 box1;
 	BOX2DFLOAT4 box2;
@@ -447,14 +450,12 @@ Datum LWGEOM_contain(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_contain --entry");
 #endif
 
-	if ( lwgeom_getsrid(lwgeom1+4) != lwgeom_getsrid(lwgeom2+4) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
 
-	if ( ! (getbox2d_p(lwgeom1+4, &box1) && getbox2d_p(lwgeom2+4, &box2)) )
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
 	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
 		PG_RETURN_BOOL(FALSE);
 	}
 
@@ -468,9 +469,12 @@ Datum LWGEOM_contain(PG_FUNCTION_ARGS)
 }
 
 
-// These functions are taken from the postgis_gist_72.c file
+/* These functions are taken from the postgis_gist_72.c file */
 
 
+/*
+ * GiST Compress methods for geometry
+ */
 PG_FUNCTION_INFO_V1(LWGEOM_gist_compress);
 Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 {
@@ -481,7 +485,7 @@ Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 	elog(NOTICE,"GIST: LWGEOM_gist_compress called");
 #endif
 
-	if ( entry->leafkey)
+	if (entry->leafkey)
 	{
 #ifdef PGIS_DEBUG_GIST4
 		elog(NOTICE,"GIST: LWGEOM_gist_compress got a leafkey");
@@ -493,67 +497,50 @@ Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 		elog(NOTICE,"GIST: LWGEOM_gist_compress got a non-NULL key");
 #endif
 
-			char *in; // lwgeom serialized
+			PG_LWGEOM *in; /* lwgeom serialized */
 			BOX2DFLOAT4 *rr;
 
-			// lwgeom serialized form
-			in = (char*)PG_DETOAST_DATUM(entry->key);
+			/* lwgeom serialized form */
+			in = (PG_LWGEOM*)PG_DETOAST_DATUM(entry->key);
 
 #ifdef PGIS_DEBUG_GIST4
 		elog(NOTICE,"GIST: LWGEOM_gist_compress detoasted entry->key: %s", unparse_WKT(in+4, malloc, free));
 #endif
 
 			if (in == NULL)
-				PG_RETURN_POINTER(entry);
-
-			// this is the EMPTY geometry
-			if (lwgeom_getnumgeometries(in+4) ==0) 
 			{
-//elog(NOTICE,"found an empty geometry");
-				// dont bother adding this to the index
+				elog(ERROR, "PG_DETOAST_DATUM(<notnull>) returned NULL ??");
 				PG_RETURN_POINTER(entry);
 			}
 
-#ifdef PGIS_DEBUG_GIST4
-		elog(NOTICE,"GIST: LWGEOM_gist_compress got numgeometries");
-#endif
-
 			rr = (BOX2DFLOAT4*) palloc(sizeof(BOX2DFLOAT4));
-			if ( ! getbox2d_p(in+4, rr) )
-			{
-#ifdef PGIS_DEBUG_GIST4
-				elog(NOTICE,
-					"found an empty geometry (cannot get box2d)");
-#endif
-				// dont bother adding this to the index
-				PG_RETURN_POINTER(entry);
-			}
-			//memcpy(rr,&r,sizeof(BOX2DFLOAT4));
 
-#ifdef PGIS_DEBUG_GIST4
-		elog(NOTICE,"GIST: LWGEOM_gist_compress got box2d");
-#endif
-
-			if ( ! finite(rr->xmin) ||
+			if (	! getbox2d_p(SERIALIZED_FORM(in), rr) ||
+				! finite(rr->xmin) ||
 				! finite(rr->ymin) ||
 				! finite(rr->xmax) ||
 				! finite(rr->ymax) )
 			{
-				elog(NOTICE, "found infinite geometry");
+#ifdef PGIS_DEBUG_GIST4
+			elog(NOTICE, "found empty or infinite geometry");
+#endif
 				pfree(rr);
+				if (in!=(PG_LWGEOM*)DatumGetPointer(entry->key))
+					pfree(in);  /* PG_FREE_IF_COPY */
 				PG_RETURN_POINTER(entry);
 			}
 
+#ifdef PGIS_DEBUG_GIST4
+			elog(NOTICE,"GIST: LWGEOM_gist_compress got box2d");
+#endif
+
 
 #ifdef PGIS_DEBUG_GIST2
 	elog(NOTICE,"GIST: LWGEOM_gist_compress -- got box2d BOX(%.15g %.15g,%.15g %.15g)", rr->xmin, rr->ymin, rr->xmax, rr->ymax);
 #endif
-			//r = convert_box3d_to_box(&in->bvol);
 
-			if ( in != (char*)DatumGetPointer(entry->key) )
-			{
-				pfree(in);  // PG_FREE_IF_COPY
-			}
+			if (in != (PG_LWGEOM*)DatumGetPointer(entry->key))
+				pfree(in);  /* PG_FREE_IF_COPY */
 
 			gistentryinit(*retval, PointerGetDatum(rr),
 				entry->rel, entry->page,
@@ -569,6 +556,7 @@ Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 			gistentryinit(*retval, (Datum) 0, entry->rel,
 				entry->page, entry->offset, 0, FALSE);
 		}
+
 	}
 	else
 	{
@@ -586,33 +574,35 @@ PG_FUNCTION_INFO_V1(LWGEOM_gist_consistent);
 Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 {
 	GISTENTRY *entry = (GISTENTRY*) PG_GETARG_POINTER(0);
-	char *query ; // lwgeom serialized form
+	PG_LWGEOM *query ; /* lwgeom serialized form */
 	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
-	// BOX		thebox;
 	bool result;
 	BOX2DFLOAT4  box;
 
-	/*
-	 * if entry is not leaf, use rtree_internal_consistent,
-	 * else use rtree_leaf_consistent
-	 */
 #ifdef PGIS_DEBUG_CALLS
 	elog(NOTICE,"GIST: LWGEOM_gist_consistent called");
 #endif
 
 	if ( ((Pointer *) PG_GETARG_DATUM(1)) == NULL )
 	{
-		//elog(NOTICE,"LWGEOM_gist_consistent:: got null query!");
-		PG_RETURN_BOOL(false); // null query - this is screwy!
+		/*elog(NOTICE,"LWGEOM_gist_consistent:: got null query!"); */
+		PG_RETURN_BOOL(false); /* null query - this is screwy! */
 	}
 
-	query = (char*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
-
+	query = (PG_LWGEOM*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	if ( ! (DatumGetPointer(entry->key) != NULL && query) )
+	{
+		PG_FREE_IF_COPY(query, 1);
+		elog(ERROR, "LWGEOM_gist_consistent got either NULL query or entry->key");
 		PG_RETURN_BOOL(FALSE);
+	}
 
-	getbox2d_p(query+4, &box);
+	if ( ! getbox2d_p(SERIALIZED_FORM(query), &box) )
+	{
+		PG_FREE_IF_COPY(query, 1);
+		PG_RETURN_BOOL(FALSE);
+	}
 
 	if (GIST_LEAF(entry))
 		result = lwgeom_rtree_leaf_consistent((BOX2DFLOAT4 *)
@@ -622,7 +612,6 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 			(BOX2DFLOAT4 *) DatumGetPointer(entry->key),
 			&box, strategy );
 
-	//pfree(thebox);
 	PG_FREE_IF_COPY(query, 1);
 	PG_RETURN_BOOL(result);
 }
@@ -639,11 +628,13 @@ lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key, BOX2DFLOAT4 *query,
 #endif
 
 	switch(strategy) {
-    case RTLeftStrategyNumber:
-    case RTOverLeftStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_overleft, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-    case RTOverlapStrategyNumber:  //optimized for speed
+		case RTLeftStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_overright, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTOverLeftStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_right, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTOverlapStrategyNumber:  /*optimized for speed */
 
         retval = (((key->xmax>= query->xmax) &&
 			 (key->xmin <= query->xmax)) ||
@@ -658,7 +649,7 @@ lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key, BOX2DFLOAT4 *query,
 
 
 #ifdef PGIS_DEBUG_GIST5
-//keep track and report info about how many times this is called
+/*keep track and report info about how many times this is called */
 					if (counter_intern == 0)
 					{
 									 elog(NOTICE,"search bounding box is: <%.16g %.16g,%.16g %.16g> - size box2d= %i",
@@ -670,33 +661,41 @@ lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key, BOX2DFLOAT4 *query,
 
 elog(NOTICE,"%i:(int)<%.8g %.8g,%.8g %.8g>&&<%.8g %.8g,%.8g %.8g> %i",counter_intern,key->xmin,key->ymin,key->xmax,key->ymax,
 		  						query->xmin,query->ymin,query->xmax,query->ymax,   (int) retval);
-#endif
 			counter_intern++;
-		  return(retval);
-      break;
-    case RTOverRightStrategyNumber:
-    case RTRightStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_overright, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-	case RTOverBelowStrategyNumber:
-	case RTBelowStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_overbelow, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-	case RTAboveStrategyNumber:
-	case RTOverAboveStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_overabove, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-    case RTSameStrategyNumber:
-    case RTContainsStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_contain, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-    case RTContainedByStrategyNumber:
-      retval = DatumGetBool( DirectFunctionCall2( BOX2D_overlap, PointerGetDatum(key), PointerGetDatum(query) ) );
-      break;
-    default:
-      retval = FALSE;
-    }
-    return(retval);
+#endif
+
+			return(retval);
+			break;
+
+		case RTOverRightStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_left, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTRightStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_overleft, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTOverBelowStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_above, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTBelowStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_overabove, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTAboveStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_overbelow, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTOverAboveStrategyNumber:
+			retval = !DatumGetBool( DirectFunctionCall2( BOX2D_below, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTSameStrategyNumber:
+		case RTContainsStrategyNumber:
+			retval = DatumGetBool( DirectFunctionCall2( BOX2D_contain, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		case RTContainedByStrategyNumber:
+			retval = DatumGetBool( DirectFunctionCall2( BOX2D_overlap, PointerGetDatum(key), PointerGetDatum(query) ) );
+			break;
+		default:
+			retval = FALSE;
+	}
+	return(retval);
 }
 
 
@@ -718,7 +717,7 @@ lwgeom_rtree_leaf_consistent(BOX2DFLOAT4 *key,
 		case RTOverLeftStrategyNumber:
 			retval = DatumGetBool(DirectFunctionCall2(BOX2D_overleft, PointerGetDatum(key), PointerGetDatum(query)));
 			break;
-		case RTOverlapStrategyNumber://optimized for speed
+		case RTOverlapStrategyNumber: /*optimized for speed */
 			           retval = (((key->xmax>= query->xmax) &&
 			   			 (key->xmin <= query->xmax)) ||
 			   			((query->xmax>= key->xmax) &&
@@ -729,7 +728,7 @@ lwgeom_rtree_leaf_consistent(BOX2DFLOAT4 *key,
 			   		 ((query->ymax>= key->ymax) &&
 			   		  (query->ymin<= key->ymax)));
 #ifdef PGIS_DEBUG_GIST5
-//keep track and report info about how many times this is called
+/*keep track and report info about how many times this is called */
 
 			   elog(NOTICE,"%i:gist test (leaf) <%.6g %.6g,%.6g %.6g> &&  <%.6g %.6g,%.6g %.6g> --> %i",counter_leaf,key->xmin,key->ymin,key->xmax,key->ymax,
 			   		  						query->xmin,query->ymin,query->xmax,query->ymax,   (int) retval);
@@ -850,9 +849,11 @@ Datum LWGEOM_gist_union(PG_FUNCTION_ARGS)
 }
 
 
-// size of a box is width*height
-// we do this in double precision because width and height can be very very small
-// and it gives screwy results
+/*
+ * size of a box is width*height
+ * we do this in double precision because width and height can be very very small
+ * and it gives screwy results
+ */
 static float
 size_box2d(Datum box2d)
 {
@@ -875,7 +876,6 @@ size_box2d(Datum box2d)
 		{
 			result = (((double) a->xmax)-((double) a->xmin)) * (((double) a->ymax)-((double) a->ymin));
 		}
-		//  result= (float) ((a->xmax - a->xmin) * (a->ymax - a->ymin));
 	}
 	else result = (float) 0.0;
 
@@ -885,10 +885,12 @@ size_box2d(Datum box2d)
 	return result;
 }
 
-// size of a box is width*height
-// we do this in double precision because width and height can be very very small
-// and it gives screwy results
-// this version returns a double
+/*
+ * size of a box is width*height
+ * we do this in double precision because width and height can be very very small
+ * and it gives screwy results
+ * this version returns a double
+ */
 static double size_box2d_double(Datum box2d);
 static double size_box2d_double(Datum box2d)
 {
@@ -902,16 +904,19 @@ static double size_box2d_double(Datum box2d)
 	{
 		BOX2DFLOAT4 *a = (BOX2DFLOAT4*) DatumGetPointer(box2d);
 
-		    if (a == (BOX2DFLOAT4 *) NULL || a->xmax <= a->xmin || a->ymax <= a->ymin)
-		        result =  (double) 0.0;
-		    else
-		    {
-				result = (((double) a->xmax)-((double) a->xmin)) * (((double) a->ymax)-((double) a->ymin));
-			}
-		      //  result= (float) ((a->xmax - a->xmin) * (a->ymax - a->ymin));
+		if (a == (BOX2DFLOAT4 *) NULL || a->xmax <= a->xmin || a->ymax <= a->ymin)
+		{
+			result =  (double) 0.0;
+		}
+		else
+		{
+			result = (((double) a->xmax)-((double) a->xmin)) * (((double) a->ymax)-((double) a->ymin));
+		}
 	}
 	else
+	{
 		result = (double) 0.0;
+	}
 
 #ifdef PGIS_DEBUG_GIST
 	elog(NOTICE,"GIST: size_box2d_double called - returning %.15g",result);
@@ -940,33 +945,15 @@ Datum LWGEOM_gist_penalty(PG_FUNCTION_ARGS)
 
 
 	ud = DirectFunctionCall2(BOX2D_union, origentry->key, newentry->key);
-	//ud = BOX2D_union(origentry->key, newentry->key);
+	/*ud = BOX2D_union(origentry->key, newentry->key); */
 	tmp1 = size_box2d_double(ud);
 	if (DatumGetPointer(ud) != NULL)
 		pfree(DatumGetPointer(ud));
 
 	*result = tmp1 - size_box2d_double(origentry->key);
-/*
-	if ( (*result <0) && (*result > -0.00001))
-		*result = 0;
-	if ( (*result >0) && (*result < 0.00001))
-		*result = 0;
-
-*/
 
 #ifdef PGIS_DEBUG_GIST6
 	elog(NOTICE,"GIST: LWGEOM_gist_penalty called and returning %.15g", *result);
-	if (*result<0)
-	{
-		BOX2DFLOAT4 *a, *b,*c;
-		a = (BOX2DFLOAT4*) DatumGetPointer(origentry->key);
-		b = (BOX2DFLOAT4*) DatumGetPointer(newentry->key);
-		//c = (BOX2DFLOAT4*) DatumGetPointer(DirectFunctionCall2(BOX2D_union, origentry->key, newentry->key));
-		c = BOX2D_union(origentry->key, newentry->key);
-		//elog(NOTICE,"LWGEOM_gist_penalty -- a = <%.16g %.16g,%.16g %.16g>", a->xmin, a->ymin, a->xmax, a->ymax);
-		//elog(NOTICE,"LWGEOM_gist_penalty -- b = <%.16g %.16g,%.16g %.16g>", b->xmin, b->ymin, b->xmax, b->ymax);
-		//elog(NOTICE,"LWGEOM_gist_penalty -- c = <%.16g %.16g,%.16g %.16g>", c->xmin, c->ymin, c->xmax, c->ymax);
-	}
 #endif
 
 	PG_RETURN_POINTER(result);
@@ -994,9 +981,9 @@ compare_KB(const void* a, const void* b) {
 PG_FUNCTION_INFO_V1(LWGEOM_gist_same);
 Datum LWGEOM_gist_same(PG_FUNCTION_ARGS)
 {
-	BOX2DFLOAT4		   *b1 = (BOX2DFLOAT4 *) PG_GETARG_POINTER(0);
-	BOX2DFLOAT4		   *b2 = (BOX2DFLOAT4 *) PG_GETARG_POINTER(1);
-	bool	   *result = (bool *) PG_GETARG_POINTER(2);
+	BOX2DFLOAT4 *b1 = (BOX2DFLOAT4 *) PG_GETARG_POINTER(0);
+	BOX2DFLOAT4 *b2 = (BOX2DFLOAT4 *) PG_GETARG_POINTER(1);
+	bool *result = (bool *)PG_GETARG_POINTER(2);
 
 #ifdef PGIS_DEBUG_CALLS
 	elog(NOTICE,"GIST: LWGEOM_gist_same called");
@@ -1011,7 +998,6 @@ Datum LWGEOM_gist_same(PG_FUNCTION_ARGS)
 
 
 
-
 /*
 ** The GiST PickSplit method
 ** New linear algorithm, see 'New Linear Node Splitting Algorithm for R-tree',
@@ -1021,36 +1007,28 @@ PG_FUNCTION_INFO_V1(LWGEOM_gist_picksplit);
 Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 {
 #if USE_VERSION < 80
-	bytea	   *entryvec = (bytea *) PG_GETARG_POINTER(0);
+	bytea *entryvec = (bytea *) PG_GETARG_POINTER(0);
 #else
   	GistEntryVector	*entryvec = (GistEntryVector *) PG_GETARG_POINTER(0);
 #endif
 	GIST_SPLITVEC *v = (GIST_SPLITVEC *) PG_GETARG_POINTER(1);
 	OffsetNumber i;
-	OffsetNumber *listL,
-			   *listR,
-			   *listB,
-			   *listT;
-	BOX2DFLOAT4		   *unionL,
-			   *unionR,
-			   *unionB,
-			   *unionT;
-	int			posL,
-				posR,
-				posB,
-				posT;
-	BOX2DFLOAT4			pageunion;
-	BOX2DFLOAT4		   *cur;
-	char		direction = ' ';
-	bool		allisequal = true;
+	OffsetNumber *listL, *listR, *listB, *listT;
+	BOX2DFLOAT4 *unionL, *unionR, *unionB, *unionT;
+	int posL, posR, posB, posT;
+	BOX2DFLOAT4 pageunion;
+	BOX2DFLOAT4 *cur;
+	char direction = ' ';
+	bool allisequal = true;
 	OffsetNumber maxoff;
-	int			nbytes;
+	int nbytes;
 
 #ifdef PGIS_DEBUG_CALLS
 	elog(NOTICE,"GIST: LWGEOM_gist_picksplit called");
 #endif
 
 	posL = posR = posB = posT = 0;
+
 #if USE_VERSION < 80
 	maxoff = ((VARSIZE(entryvec) - VARHDRSZ) / sizeof(GISTENTRY)) - 1;
 	cur = (BOX2DFLOAT4*) DatumGetPointer(((GISTENTRY *) VARDATA(entryvec))[FirstOffsetNumber].key);
@@ -1145,8 +1123,6 @@ elog(NOTICE," AllIsEqual!");
 	}
 
 
-
-
 	listB = (OffsetNumber *) palloc(nbytes);
 	listT = (OffsetNumber *) palloc(nbytes);
 	unionB = (BOX2DFLOAT4 *) palloc(sizeof(BOX2DFLOAT4));
@@ -1242,11 +1218,16 @@ elog(NOTICE,"   unionB is: <%.16g %.16g,%.16g %.16g>", unionB->xmin, unionB->ymi
 			PointerGetDatum(unionB), PointerGetDatum(unionT));
 		float sizeLR, sizeBT;
 
-//elog(NOTICE,"direction is abigeous");
+/*elog(NOTICE,"direction is abigeous"); */
 
 		sizeLR = size_box2d(interLR);
 		sizeBT = size_box2d(interBT);
 
+		if ( DatumGetPointer(interLR) != NULL )
+			pfree(DatumGetPointer(interLR));
+		if ( DatumGetPointer(interBT) != NULL )
+			pfree(DatumGetPointer(interBT));
+
 		if (sizeLR < sizeBT)
 			direction = 'x';
 		else
@@ -1268,29 +1249,29 @@ elog(NOTICE,"   unionB is: <%.16g %.16g,%.16g %.16g>", unionB->xmin, unionB->ymi
 		v->spl_rdatum = PointerGetDatum(unionR);
 
 #ifdef PGIS_DEBUG_GIST6
-elog(NOTICE,"   split direction was '%c'", direction);
-elog(NOTICE,"   posL = %i, posR=%i", posL,posR);
-elog(NOTICE,"   posL's (nleft) offset numbers:");
-{
-		char		aaa[5000],bbb[100];
-		aaa[0] =0;
+	{
+		char aaa[5000],bbb[100];
+		aaa[0] = 0;
 
-for (i=0;i<posL;i++)
-{
-	sprintf(bbb," %i", listL[i]);
-	strcat(aaa,bbb);
-}
-elog(NOTICE,aaa);
-aaa[0]=0;
-elog(NOTICE,"   posR's (nright) offset numbers:");
-for (i=0;i<posR;i++)
-{
-	sprintf(bbb," %i", listR[i]);
-	strcat(aaa,bbb);
-}
-elog(NOTICE,aaa);
-}
+		elog(NOTICE,"   split direction was '%c'", direction);
+		elog(NOTICE,"   posL = %i, posR=%i", posL,posR);
+		elog(NOTICE,"   posL's (nleft) offset numbers:");
 
+		for (i=0;i<posL;i++)
+		{
+			sprintf(bbb," %i", listL[i]);
+			strcat(aaa,bbb);
+		}
+		elog(NOTICE,aaa);
+		aaa[0]=0;
+		elog(NOTICE,"   posR's (nright) offset numbers:");
+		for (i=0;i<posR;i++)
+		{
+			sprintf(bbb," %i", listR[i]);
+			strcat(aaa,bbb);
+		}
+		elog(NOTICE,aaa);
+	}
 #endif
 
 
@@ -1310,27 +1291,29 @@ elog(NOTICE,aaa);
 		v->spl_rdatum = PointerGetDatum(unionT);
 
 #ifdef PGIS_DEBUG_GIST6
-elog(NOTICE,"   split direction was '%c'", direction);
-elog(NOTICE,"   posB = %i, posT=%i", posB,posT);
-elog(NOTICE,"   posB's (nleft) offset numbers:");
-{
-	char		aaa[5000],bbb[100];
-	aaa[0] =0;
-for (i=0;i<posB;i++)
-{
-	sprintf(bbb," %i", listB[i]);
-	strcat(aaa,bbb);
-}
-elog(NOTICE,aaa);
-aaa[0]=0;
-elog(NOTICE,"   posT's (nright) offset numbers:");
-for (i=0;i<posT;i++)
-{
-	sprintf(bbb," %i", listT[i]);
-	strcat(aaa,bbb);
-}
-elog(NOTICE,aaa);
-}
+	{
+		char aaa[5000],bbb[100];
+		aaa[0]=0;
+
+		elog(NOTICE,"   split direction was '%c'", direction);
+		elog(NOTICE,"   posB = %i, posT=%i", posB,posT);
+		elog(NOTICE,"   posB's (nleft) offset numbers:");
+
+		for (i=0;i<posB;i++)
+		{
+			sprintf(bbb," %i", listB[i]);
+			strcat(aaa,bbb);
+		}
+		elog(NOTICE,aaa);
+		aaa[0]=0;
+		elog(NOTICE,"   posT's (nright) offset numbers:");
+		for (i=0;i<posT;i++)
+		{
+			sprintf(bbb," %i", listT[i]);
+			strcat(aaa,bbb);
+		}
+		elog(NOTICE,aaa);
+	}
 #endif
 
 
@@ -1339,13 +1322,13 @@ elog(NOTICE,aaa);
 }
 
 
-// debug function
+/* debug function */
 Datum report_lwgeom_gist_activity(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(report_lwgeom_gist_activity);
 Datum report_lwgeom_gist_activity(PG_FUNCTION_ARGS)
 {
 	elog(NOTICE,"lwgeom gist activity - internal consistency= %i, leaf consistency = %i",counter_intern,counter_leaf);
-	counter_intern =0;  //reset
+	counter_intern =0;  /*reset */
 	counter_leaf = 0;
 	PG_RETURN_NULL();
 }
