@@ -34,7 +34,10 @@ Datum pointonsurface(PG_FUNCTION_ARGS);
 Datum JTSnoop(PG_FUNCTION_ARGS);
 Datum postgis_geos_version(PG_FUNCTION_ARGS);
 Datum centroid(PG_FUNCTION_ARGS);
-Datum JTS_polygonize_garray(PG_FUNCTION_ARGS);
+Datum polygonize_garray(PG_FUNCTION_ARGS);
+Datum polygonize_garray(PG_FUNCTION_ARGS);
+Datum LWGEOM_buildarea(PG_FUNCTION_ARGS);
+Datum linemerge(PG_FUNCTION_ARGS);
 
 
 
@@ -101,7 +104,8 @@ extern int      JTSGetNumCoordinate(JTSGeometry *g1);
 extern JTSGeometry	*JTSGetGeometryN(JTSGeometry *g1, int n);
 extern JTSGeometry	*JTSGetExteriorRing(JTSGeometry *g1);
 extern JTSGeometry	*JTSGetInteriorRingN(JTSGeometry *g1,int n);
-extern JTSGeometry *JTSpolygonize(JTSGeometry **geoms, unsigned int ngeoms);
+extern JTSGeometry *JTSPolygonize(JTSGeometry **geoms, unsigned int ngeoms);
+extern JTSGeometry *JTSLineMerge(JTSGeometry *geoms);
 extern int	JTSGetNumInteriorRings(JTSGeometry *g1);
 extern int      JTSGetSRID(JTSGeometry *g1);
 extern int      JTSGetNumGeometries(JTSGeometry *g1);
@@ -127,9 +131,15 @@ LWGEOM *JTS2LWGEOM(JTSGeometry *g, char want3d);
 
 
 /*
- * This is the final function for union/fastunite/geomunion
- * aggregate (still discussing the name). Will have
- * as input an array of JTSGeometry objects.
+ * Using BUFFER(0) version of unite_garray takes
+ * processing to a segfault (after spending a lot of time!)
+ */
+#undef UNITE_USING_BUFFER 
+
+#ifndef UNITE_USING_BUFFER
+/*
+ * This is the final function for geomunion
+ * aggregate. Will have as input an array of Geometries.
  * Will iteratively call JTSUnion on the JTS-converted
  * versions of them and return PGIS-converted version back.
  * Changing combination order *might* speed up performance.
@@ -154,7 +164,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 	call++;
 #endif
 
-	//lwnotice("unite_garray (jts) invoked");
+	lwnotice("JTS incremental union");
 
 	datum = PG_GETARG_DATUM(0);
 
@@ -206,12 +216,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 		}
 		else
 		{
-			if ( SRID != pglwgeom_getSRID(geom) )
-			{
-				finishJTS();
-				lwerror("Operation on mixed SRID geometries");
-				PG_RETURN_NULL();
-			}
+			errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
 		}
 		
 		g1 = POSTGIS2JTS(pgis_geom);
@@ -254,6 +259,113 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 
 }
 
+#else // def UNITE_USING_BUFFER
+
+/*
+ * This is the final function for GeomUnion
+ * aggregate. Will have as input an array of Geometries.
+ * Builds a GEOMETRYCOLLECTION from input and call
+ * JTSBuffer(collection, 0) on the JTS-converted
+ * versions of it. Returns PGIS-converted version back.
+ */
+PG_FUNCTION_INFO_V1(unite_garray);
+Datum unite_garray(PG_FUNCTION_ARGS)
+{
+	Datum datum;
+	ArrayType *array;
+	int is3d = 0;
+	int nelems, i;
+	PG_LWGEOM *result, *pgis_geom;
+	LWGEOM **lwgeoms, *collection;
+	JTSGeometry *g1, *geos_result=NULL;
+	int SRID=-1;
+	size_t offset;
+#ifdef PGIS_DEBUG
+	static int call=1;
+#endif
+
+#ifdef PGIS_DEBUG
+	call++;
+#endif
+
+	lwnotice("JTS buffer union");
+
+	datum = PG_GETARG_DATUM(0);
+
+	/* Null array, null geometry (should be empty?) */
+	if ( (Pointer *)datum == NULL ) PG_RETURN_NULL();
+
+	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+
+	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
+
+#ifdef PGIS_DEBUG
+	elog(NOTICE, "unite_garray: number of elements: %d", nelems);
+#endif
+
+	if ( nelems == 0 ) PG_RETURN_NULL();
+
+	/* One-element union is the element itself */
+	if ( nelems == 1 ) PG_RETURN_POINTER((PG_LWGEOM *)(ARR_DATA_PTR(array)));
+
+	lwgeoms = lwalloc(sizeof(LWGEOM *)*nelems);
+
+	offset = 0;
+	for (i=0; i<nelems; i++)
+	{
+		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
+		offset += INTALIGN(geom->size);
+
+		pgis_geom = geom;
+
+		// Check is3d flag
+		if ( TYPE_NDIMS(geom->type) > 2 ) is3d = 1;
+
+		// Check SRID homogeneity 
+		if ( ! i ) SRID = pglwgeom_getSRID(geom);
+		else errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
+		
+		lwgeoms[i] = lwgeom_deserialize(SERIALIZED_FORM(pgis_geom));
+
+	}
+
+	/* Let's build a GEOMETRYCOLLECTION */
+	collection = (LWGEOM *)lwcollection_construct(COLLECTIONTYPE, -1,
+		NULL, nelems, lwgeoms);
+
+	/* Ok, we really need JTS now */
+	initJTS(lwnotice);
+
+	g1 = LWGEOM2JTS(collection);
+
+	/* we don't need these anymore */
+	lwgeom_release(collection);
+
+	geos_result = JTSBuffer(g1, 0, 0);
+	g1=NULL;
+	if ( geos_result == NULL )
+	{
+		finishJTS();
+		lwerror("GEOS buffer() threw an error!");
+	}
+	JTSSetSRID(geos_result, SRID);
+
+	result = JTS2POSTGIS(geos_result, is3d);
+	geos_result=NULL;
+
+	finishJTS();
+
+	if ( result == NULL )
+		lwerror("JTS2POSTGIS returned an error");
+
+	//compressType(result);
+
+	PG_RETURN_POINTER(result);
+
+}
+
+#endif // def UNITE_USING_BUFFER
+
 
 //select geomunion('POLYGON((0 0, 10 0, 10 10, 0 10, 0 0))','POLYGON((5 5, 15 5, 15 7, 5 7, 5 5))');
 PG_FUNCTION_INFO_V1(geomunion);
@@ -277,11 +389,7 @@ Datum geomunion(PG_FUNCTION_ARGS)
 		( TYPE_NDIMS(geom2->type) > 2 );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initJTS(lwnotice);
 //elog(NOTICE,"in geomunion");
@@ -341,7 +449,7 @@ Datum geomunion(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, result);
+	profreport("jts",geom1, geom2, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -374,11 +482,7 @@ Datum symdifference(PG_FUNCTION_ARGS)
 		( TYPE_NDIMS(geom2->type) > 2 );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initJTS(lwnotice);
 
@@ -433,7 +537,7 @@ Datum symdifference(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, result);
+	profreport("jts",geom1, geom2, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -509,7 +613,7 @@ Datum boundary(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, NULL, result);
+	profreport("jts",geom1, NULL, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -586,9 +690,7 @@ Datum convexhull(PG_FUNCTION_ARGS)
 	/* Copy input bbox if any */
 	if ( getbox2d_p(SERIALIZED_FORM(geom1), &bbox) )
 	{
-		lwout->bbox = &bbox;
-		/* Mark lwgeom bbox to be externally owned */
-		TYPE_SETHASBBOX(lwout->type, 1);
+		lwout->bbox = box2d_clone(&bbox);
 	}
 
 	result = pglwgeom_serialize(lwout);
@@ -604,7 +706,7 @@ Datum convexhull(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, NULL, result);
+	profreport("jts",geom1, NULL, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -681,7 +783,7 @@ Datum buffer(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, NULL, result);
+	profreport("jts",geom1, NULL, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -710,11 +812,7 @@ Datum intersection(PG_FUNCTION_ARGS)
 		( TYPE_NDIMS(geom2->type) > 2 );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initJTS(lwnotice);
 
@@ -794,7 +892,7 @@ Datum intersection(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, result);
+	profreport("jts",geom1, geom2, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -825,11 +923,7 @@ Datum difference(PG_FUNCTION_ARGS)
 		( TYPE_NDIMS(geom2->type) > 2 );
 
 	SRID = pglwgeom_getSRID(geom1);
-	if ( SRID != pglwgeom_getSRID(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+	errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom2));
 
 	initJTS(lwnotice);
 
@@ -886,7 +980,7 @@ Datum difference(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, result);
+	profreport("jts",geom1, geom2, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -956,7 +1050,7 @@ Datum pointonsurface(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, NULL, result);
+	profreport("jts",geom1, NULL, result);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1024,7 +1118,7 @@ Datum centroid(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom, NULL, result);
+	profreport("jts",geom, NULL, result);
 #endif
 
 	PG_FREE_IF_COPY(geom, 0);
@@ -1104,7 +1198,7 @@ Datum isvalid(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, NULL, NULL);
+	profreport("jts",geom1, NULL, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1133,6 +1227,7 @@ Datum overlaps(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1182,7 +1277,7 @@ Datum overlaps(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1210,6 +1305,7 @@ Datum contains(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box is not completely inside
@@ -1260,7 +1356,7 @@ Datum contains(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1287,6 +1383,7 @@ Datum within(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom1 bounding box is not completely inside
@@ -1337,7 +1434,7 @@ Datum within(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1365,6 +1462,7 @@ Datum crosses(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1415,7 +1513,7 @@ Datum crosses(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1443,6 +1541,7 @@ Datum intersects(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1518,6 +1617,7 @@ Datum touches(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1568,7 +1668,7 @@ Datum touches(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1595,6 +1695,7 @@ Datum disjoint(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not overlap
@@ -1645,7 +1746,7 @@ Datum disjoint(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1672,6 +1773,7 @@ Datum relate_pattern(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	initJTS(lwnotice);
 
@@ -1712,7 +1814,7 @@ Datum relate_pattern(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1743,6 +1845,7 @@ Datum relate_full(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 
 	initJTS(lwnotice);
@@ -1812,7 +1915,7 @@ Datum relate_full(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1840,6 +1943,7 @@ Datum geomequals(PG_FUNCTION_ARGS)
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
 
 	errorIfJTSGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
 
 	/*
 	 * short-circuit 1: if geom2 bounding box does not equal
@@ -1890,7 +1994,7 @@ Datum geomequals(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("jts",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1945,7 +2049,7 @@ Datum issimple(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom, NULL, NULL);
+	profreport("jts",geom, NULL, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom, 0);
@@ -2004,7 +2108,7 @@ Datum isring(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom, NULL, NULL);
+	profreport("jts",geom, NULL, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom, 0);
@@ -2483,8 +2587,8 @@ Datum JTSnoop(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
-PG_FUNCTION_INFO_V1(JTS_polygonize_garray);
-Datum JTS_polygonize_garray(PG_FUNCTION_ARGS)
+PG_FUNCTION_INFO_V1(polygonize_garray);
+Datum polygonize_garray(PG_FUNCTION_ARGS)
 {
 	Datum datum;
 	ArrayType *array;
@@ -2513,7 +2617,7 @@ Datum JTS_polygonize_garray(PG_FUNCTION_ARGS)
 	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "JTS_polygonize_garray: number of elements: %d", nelems);
+	elog(NOTICE, "polygonize_garray: number of elements: %d", nelems);
 #endif
 
 	if ( nelems == 0 ) PG_RETURN_NULL();
@@ -2545,12 +2649,12 @@ Datum JTS_polygonize_garray(PG_FUNCTION_ARGS)
 	}
 
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "JTS_polygonize_garray: invoking JTSpolygonize");
+	elog(NOTICE, "polygonize_garray: invoking JTSpolygonize");
 #endif
 
-	geos_result = JTSpolygonize(vgeoms, nelems);
+	geos_result = JTSPolygonize(vgeoms, nelems);
 #ifdef PGIS_DEBUG
-	elog(NOTICE, "JTS_polygonize_garray: JTSpolygonize returned");
+	elog(NOTICE, "polygonize_garray: JTSpolygonize returned");
 #endif
 	//pfree(vgeoms);
 	if ( ! geos_result ) {
@@ -2573,19 +2677,78 @@ Datum JTS_polygonize_garray(PG_FUNCTION_ARGS)
 
 }
 
-Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS);
-PG_FUNCTION_INFO_V1(GEOS_polygonize_garray);
-Datum GEOS_polygonize_garray(PG_FUNCTION_ARGS)
+PG_FUNCTION_INFO_V1(linemerge);
+Datum linemerge(PG_FUNCTION_ARGS)
 {
-	elog(NOTICE, "GEOS support is disabled, use JTS_polygonize_garray instead");
-	PG_RETURN_NULL();
+	PG_LWGEOM	*geom1;
+	JTSGeometry *g1,*g3;
+	PG_LWGEOM *result;
+
+#ifdef PROFILE
+	profstart(PROF_QRUN);
+#endif
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+	initJTS(lwnotice);
+
+#ifdef PROFILE
+	profstart(PROF_P2G1);
+#endif
+	g1 = POSTGIS2JTS(geom1);
+#ifdef PROFILE
+	profstop(PROF_P2G1);
+#endif
+
+#ifdef PROFILE
+	profstart(PROF_GRUN);
+#endif
+	g3 = JTSLineMerge(g1);
+#ifdef PROFILE
+	profstop(PROF_GRUN);
+#endif
+
+	if (g3 == NULL)
+	{
+		elog(ERROR,"JTS LineMerge() threw an error!");
+		PG_RETURN_NULL(); //never get here
+	}
+
+
+//	elog(NOTICE,"result: %s", GEOSasText(g3) ) ;
+
+	JTSSetSRID(g3, pglwgeom_getSRID(geom1));
+
+#ifdef PROFILE
+	profstart(PROF_G2P);
+#endif
+	result = JTS2POSTGIS(g3, TYPE_NDIMS(geom1->type) > 2);
+#ifdef PROFILE
+	profstop(PROF_G2P);
+#endif
+	if (result == NULL)
+	{
+		elog(ERROR,"JTS LineMerge() threw an error (result postgis geometry formation)!");
+		PG_RETURN_NULL(); //never get here
+	}
+
+	//compressType(result);  // convert multi* to single item if appropriate
+
+#ifdef PROFILE
+	profstop(PROF_QRUN);
+	profreport("jts",geom1, NULL, result);
+#endif
+
+	PG_FREE_IF_COPY(geom1, 0);
+
+	PG_RETURN_POINTER(result);
 }
 
 Datum GEOSnoop(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(GEOSnoop);
 Datum GEOSnoop(PG_FUNCTION_ARGS)
 {
-	elog(NOTICE, "GEOS support is disabled, use JTSnoop instead");
+	elog(ERROR, "GEOS support is disabled");
 	PG_RETURN_NULL();
 }
 
@@ -2595,3 +2758,28 @@ Datum postgis_geos_version(PG_FUNCTION_ARGS)
 	PG_RETURN_NULL();
 }
 
+PG_FUNCTION_INFO_V1(postgis_jts_version);
+Datum postgis_jts_version(PG_FUNCTION_ARGS)
+{
+	char *ver;
+	text *result;
+
+	initJTS(lwnotice);
+
+	ver = JTSversion();
+	result = (text *) palloc(VARHDRSZ  + strlen(ver));
+	VARATT_SIZEP(result) = VARHDRSZ + strlen(ver) ;
+	memcpy(VARDATA(result), ver, strlen(ver));
+	free(ver);
+
+	finishJTS();
+
+	PG_RETURN_POINTER(result);
+}
+
+PG_FUNCTION_INFO_V1(LWGEOM_buildarea);
+Datum LWGEOM_buildarea(PG_FUNCTION_ARGS)
+{
+	elog(ERROR,"BuildArea: operation not implemented in JTS wrapper - compile PostGIS with GEOS support");
+	PG_RETURN_NULL(); // never get here
+}
