@@ -3,18 +3,20 @@
 
 #include "liblwgeom.h"
 
-//#define PGIS_DEBUG
+/*#define PGIS_DEBUG*/
 
-// pt_in_ring_2d(): crossing number test for a point in a polygon
-//      input:   p = a point,
-//               pa = vertex points of a ring V[n+1] with V[n]=V[0]
-//      returns: 0 = outside, 1 = inside
-//
-//	Our polygons have first and last point the same,
-//
+/*
+ * pt_in_ring_2d(): crossing number test for a point in a polygon
+ *      input:   p = a point,
+ *               pa = vertex points of a ring V[n+1] with V[n]=V[0]
+ *      returns: 0 = outside, 1 = inside
+ *
+ *	Our polygons have first and last point the same,
+ *
+ */
 int pt_in_ring_2d(POINT2D *p, POINTARRAY *ring)
 {
-	int cn = 0;    // the crossing number counter
+	int cn = 0;    /* the crossing number counter */
 	int i;
 	POINT2D v1, v2;
 
@@ -36,29 +38,29 @@ int pt_in_ring_2d(POINT2D *p, POINTARRAY *ring)
 	printPA(ring);
 #endif
 
-	// loop through all edges of the polygon
+	/* loop through all edges of the polygon */
 	getPoint2d_p(ring, 0, &v1);
     	for (i=0; i<ring->npoints-1; i++)
 	{   
 		double vt;
 		getPoint2d_p(ring, i+1, &v2);
 
-		// edge from vertex i to vertex i+1
+		/* edge from vertex i to vertex i+1 */
        		if
 		(
-			// an upward crossing
+			/* an upward crossing */
 			((v1.y <= p->y) && (v2.y > p->y))
-			// a downward crossing
+			/* a downward crossing */
        		 	|| ((v1.y > p->y) && (v2.y <= p->y))
 		)
 	 	{
 
 			vt = (double)(p->y - v1.y) / (v2.y - v1.y);
 
-			// P.x <intersect
+			/* P.x <intersect */
 			if (p->x < v1.x + vt * (v2.x - v1.x))
 			{
-				// a valid crossing of y=p.y right of p.x
+				/* a valid crossing of y=p.y right of p.x */
            	     		++cn;
 			}
 		}
@@ -67,7 +69,7 @@ int pt_in_ring_2d(POINT2D *p, POINTARRAY *ring)
 #ifdef PGIS_DEBUG
 	lwnotice("pt_in_ring_2d returning %d", cn&1);
 #endif
-	return (cn&1);    // 0 if even (out), and 1 if odd (in)
+	return (cn&1);    /* 0 if even (out), and 1 if odd (in) */
 }
 
 double distance2d_pt_pt(POINT2D *p1, POINT2D *p2)
@@ -77,27 +79,29 @@ double distance2d_pt_pt(POINT2D *p1, POINT2D *p2)
 		); 
 }
 
-//distance2d from p to line A->B
+/*distance2d from p to line A->B */
 double distance2d_pt_seg(POINT2D *p, POINT2D *A, POINT2D *B)
 {
 	double	r,s;
 
-	//if start==end, then use pt distance
+	/*if start==end, then use pt distance */
 	if (  ( A->x == B->x) && (A->y == B->y) )
 		return distance2d_pt_pt(p,A);
 
-	//otherwise, we use comp.graphics.algorithms Frequently Asked Questions method
-
-	/*(1)     	      AC dot AB
-                   r = ---------
-                         ||AB||^2
-		r has the following meaning:
-		r=0 P = A
-		r=1 P = B
-		r<0 P is on the backward extension of AB
-		r>1 P is on the forward extension of AB
-		0<r<1 P is interior to AB
-	*/
+	/*
+	 * otherwise, we use comp.graphics.algorithms
+	 * Frequently Asked Questions method
+	 *
+	 *  (1)     	      AC dot AB
+         *         r = ---------
+         *               ||AB||^2
+	 *	r has the following meaning:
+	 *	r=0 P = A
+	 *	r=1 P = B
+	 *	r<0 P is on the backward extension of AB
+	 *	r>1 P is on the forward extension of AB
+	 *	0<r<1 P is interior to AB
+	 */
 
 	r = ( (p->x-A->x) * (B->x-A->x) + (p->y-A->y) * (B->y-A->y) )/( (B->x-A->x)*(B->x-A->x) +(B->y-A->y)*(B->y-A->y) );
 
@@ -105,14 +109,15 @@ double distance2d_pt_seg(POINT2D *p, POINT2D *A, POINT2D *B)
 	if (r>1) return distance2d_pt_pt(p,B);
 
 
-	/*(2)
-		     (Ay-Cy)(Bx-Ax)-(Ax-Cx)(By-Ay)
-		s = -----------------------------
-		             	L^2
-
-		Then the distance from C to P = |s|*L.
-
-	*/
+	/*
+	 * (2)
+	 *	     (Ay-Cy)(Bx-Ax)-(Ax-Cx)(By-Ay)
+	 *	s = -----------------------------
+	 *	             	L^2
+	 *
+	 *	Then the distance from C to P = |s|*L.
+	 *
+	 */
 
 	s = ( (A->y-p->y)*(B->x-A->x)- (A->x-p->x)*(B->y-A->y) ) /
 		( (B->x-A->x)*(B->x-A->x) +(B->y-A->y)*(B->y-A->y) );
@@ -122,7 +127,7 @@ double distance2d_pt_seg(POINT2D *p, POINT2D *A, POINT2D *B)
 		);
 }
 
-// find the minimum 2d distance from AB to CD
+/* find the minimum 2d distance from AB to CD */
 double distance2d_seg_seg(POINT2D *A, POINT2D *B, POINT2D *C, POINT2D *D)
 {
 
@@ -135,16 +140,16 @@ double distance2d_seg_seg(POINT2D *A, POINT2D *B, POINT2D *C, POINT2D *D)
 #endif
 
 
-	//A and B are the same point
+	/*A and B are the same point */
 	if (  ( A->x == B->x) && (A->y == B->y) )
 		return distance2d_pt_seg(A,C,D);
 
-		//U and V are the same point
+		/*U and V are the same point */
 
 	if (  ( C->x == D->x) && (C->y == D->y) )
 		return distance2d_pt_seg(D,A,B);
 
-	// AB and CD are line segments
+	/* AB and CD are line segments */
 	/* from comp.graphics.algo
 
 	Solving the above for r and s yields
@@ -188,7 +193,7 @@ double distance2d_seg_seg(POINT2D *A, POINT2D *B, POINT2D *C, POINT2D *D)
 
 	if ((r<0) || (r>1) || (s<0) || (s>1) )
 	{
-		//no intersection
+		/*no intersection */
 		return (
 			LW_MIN(distance2d_pt_seg(A,C,D),
 				LW_MIN(distance2d_pt_seg(B,C,D),
@@ -200,12 +205,14 @@ double distance2d_seg_seg(POINT2D *A, POINT2D *B, POINT2D *C, POINT2D *D)
 
 	}
 	else
-		return -0; //intersection exists
+		return -0; /*intersection exists */
 
 }
 
-//search all the segments of pointarray to see which one is closest to p1
-//Returns minimum distance between point and pointarray
+/*
+ * search all the segments of pointarray to see which one is closest to p1
+ * Returns minimum distance between point and pointarray
+ */
 double distance2d_pt_ptarray(POINT2D *p, POINTARRAY *pa)
 {
 	double result = 0;
@@ -230,11 +237,11 @@ double distance2d_pt_ptarray(POINT2D *p, POINTARRAY *pa)
 	return result;
 }
 
-// test each segment of l1 against each segment of l2.  Return min
+/* test each segment of l1 against each segment of l2.  Return min */
 double distance2d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2)
 {
 	double 	result = 99999999999.9;
-	char result_okay = 0; //result is a valid min
+	char result_okay = 0; /*result is a valid min */
 	int t,u;
 	POINT2D	start, end;
 	POINT2D	start2, end2;
@@ -245,19 +252,21 @@ double distance2d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2)
 #endif
 
 	getPoint2d_p(l1, 0, &start);
-	for (t=1; t<l1->npoints; t++) //for each segment in L1
+	for (t=1; t<l1->npoints; t++) /*for each segment in L1 */
 	{
 		getPoint2d_p(l1, t, &end);
 
 		getPoint2d_p(l2, 0, &start2);
-		for (u=1; u<l2->npoints; u++) //for each segment in L2
+		for (u=1; u<l2->npoints; u++) /*for each segment in L2 */
 		{
 			double dist;
 
 			getPoint2d_p(l2, u, &end2);
 
 			dist = distance2d_seg_seg(&start, &end, &start2, &end2);
-//printf("line_line; seg %i * seg %i, dist = %g\n",t,u,dist_this);
+#if PGIS_DEBUG > 1
+printf("line_line; seg %i * seg %i, dist = %g\n",t,u,dist_this);
+#endif
 
 			if (result_okay)
 				result = LW_MIN(result,dist);
@@ -272,7 +281,7 @@ double distance2d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2)
 				t, u, dist, result);
 #endif
 
-			if (result <= 0) return 0; //intersection
+			if (result <= 0) return 0; /*intersection */
 
 			start2 = end2;
 		}
@@ -282,32 +291,34 @@ double distance2d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2)
 	return result;
 }
 
-// true if point is in poly (and not in its holes)
+/* true if point is in poly (and not in its holes) */
 int pt_in_poly_2d(POINT2D *p, LWPOLY *poly)
 {
 	int i;
 
-	// Not in outer ring
+	/* Not in outer ring */
 	if ( ! pt_in_ring_2d(p, poly->rings[0]) ) return 0;
 
-	// Check holes
+	/* Check holes */
 	for (i=1; i<poly->nrings; i++)
 	{
-		// Inside a hole
+		/* Inside a hole */
 		if ( pt_in_ring_2d(p, poly->rings[i]) ) return 0;
 	}
 
-	return 1; // In outer ring, not in holes
+	return 1; /* In outer ring, not in holes */
 }
 
-// Brute force.
-// Test line-ring distance against each ring.
-// If there's an intersection (distance==0) then return 0 (crosses boundary).
-// Otherwise, test to see if any point is inside outer rings of polygon,
-// but not in inner rings.
-// If so, return 0  (line inside polygon),
-// otherwise return min distance to a ring (could be outside
-// polygon or inside a hole)
+/*
+ * Brute force.
+ * Test line-ring distance against each ring.
+ * If there's an intersection (distance==0) then return 0 (crosses boundary).
+ * Otherwise, test to see if any point is inside outer rings of polygon,
+ * but not in inner rings.
+ * If so, return 0  (line inside polygon),
+ * otherwise return min distance to a ring (could be outside
+ * polygon or inside a hole)
+ */
 double distance2d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly)
 {
 	POINT2D pt;
@@ -328,30 +339,39 @@ double distance2d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly)
 			i, dist, mindist);
 #endif
 
-		if ( mindist <= 0 ) return 0.0; // intersection
+		if ( mindist <= 0 ) return 0.0; /* intersection */
 	}
 
-	// No intersection, have to check if a point is
-	// inside polygon
+	/*
+	 * No intersection, have to check if a point is
+	 * inside polygon
+	 */
 	getPoint2d_p(pa, 0, &pt);
 
-	// Outside outer ring, so min distance to a ring
-	// is the actual min distance
+	/*
+	 * Outside outer ring, so min distance to a ring
+	 * is the actual min distance
+	 */
 	if ( ! pt_in_ring_2d(&pt, poly->rings[0]) ) return mindist;
 
-	// Its in the outer ring.
-	// Have to check if its inside a hole
+
+	/*
+	 * Its in the outer ring.
+	 * Have to check if its inside a hole
+	 */
 	for (i=1; i<poly->nrings; i++)
 	{
 		if ( pt_in_ring_2d(&pt, poly->rings[i]) )
 		{
-			// Its inside a hole, then the actual
-			// distance is the min ring distance
+			/*
+			 * Its inside a hole, then the actual
+			 * distance is the min ring distance
+			 */
 			return mindist;
 		}
 	}
 
-	return 0.0; // Not in hole, so inside polygon
+	return 0.0; /* Not in hole, so inside polygon */
 }
 
 double distance2d_point_point(LWPOINT *point1, LWPOINT *point2)
@@ -380,9 +400,11 @@ double distance2d_line_line(LWLINE *line1, LWLINE *line2)
 	return distance2d_ptarray_ptarray(pa1, pa2);
 }
 
-// 1. see if pt in outer boundary. if no, then treat the outer ring like a line
-// 2. if in the boundary, test to see if its in a hole.
-//    if so, then return dist to hole, else return 0 (point in polygon)
+/*
+ * 1. see if pt in outer boundary. if no, then treat the outer ring like a line
+ * 2. if in the boundary, test to see if its in a hole.
+ *    if so, then return dist to hole, else return 0 (point in polygon)
+ */
 double distance2d_point_poly(LWPOINT *point, LWPOLY *poly)
 {
 	POINT2D p;
@@ -394,7 +416,7 @@ double distance2d_point_poly(LWPOINT *point, LWPOLY *poly)
 	lwnotice("distance2d_point_poly called");
 #endif
 
-	// Return distance to outer ring if not inside it
+	/* Return distance to outer ring if not inside it */
 	if ( ! pt_in_ring_2d(&p, poly->rings[0]) )
 	{
 #ifdef PGIS_DEBUG
@@ -403,13 +425,15 @@ double distance2d_point_poly(LWPOINT *point, LWPOLY *poly)
 		return distance2d_pt_ptarray(&p, poly->rings[0]);
 	}
 
-	// Inside the outer ring.
-	// Scan though each of the inner rings looking to
-	// see if its inside.  If not, distance==0.
-	// Otherwise, distance = pt to ring distance
+	/*
+	 * Inside the outer ring.
+	 * Scan though each of the inner rings looking to
+	 * see if its inside.  If not, distance==0.
+	 * Otherwise, distance = pt to ring distance
+	 */
 	for (i=1; i<poly->nrings; i++) 
 	{
-		// Inside a hole. Distance = pt -> ring
+		/* Inside a hole. Distance = pt -> ring */
 		if ( pt_in_ring_2d(&p, poly->rings[i]) )
 		{
 #ifdef PGIS_DEBUG
@@ -422,14 +446,16 @@ double distance2d_point_poly(LWPOINT *point, LWPOLY *poly)
 #ifdef PGIS_DEBUG
 	lwnotice(" inside the polygon");
 #endif
-	return 0.0; // Is inside the polygon
+	return 0.0; /* Is inside the polygon */
 }
 
-// Brute force.
-// Test to see if any rings intersect.
-// If yes, dist=0.
-// Test to see if one inside the other and if they are inside holes.
-// Find min distance ring-to-ring.
+/*
+ * Brute force.
+ * Test to see if any rings intersect.
+ * If yes, dist=0.
+ * Test to see if one inside the other and if they are inside holes.
+ * Find min distance ring-to-ring.
+ */
 double distance2d_poly_poly(LWPOLY *poly1, LWPOLY *poly2)
 {
 	POINT2D pt;
@@ -440,13 +466,11 @@ double distance2d_poly_poly(LWPOLY *poly1, LWPOLY *poly2)
 	lwnotice("distance2d_poly_poly called");
 #endif
 
-	// if poly1 inside poly2 return 0
-	//pt = (POINT2D *)getPoint(poly1->rings[0], 0);
+	/* if poly1 inside poly2 return 0 */
 	getPoint2d_p(poly1->rings[0], 0, &pt);
 	if ( pt_in_poly_2d(&pt, poly2) ) return 0.0;  
 
-	// if poly2 inside poly1 return 0
-	//pt = (POINT2D *)getPoint(poly2->rings[0], 0);
+	/* if poly2 inside poly1 return 0 */
 	getPoint2d_p(poly2->rings[0], 0, &pt);
 	if ( pt_in_poly_2d(&pt, poly1) ) return 0.0;  
 
@@ -454,23 +478,31 @@ double distance2d_poly_poly(LWPOLY *poly1, LWPOLY *poly2)
 	lwnotice("  polys not inside each other");
 #endif
 
-	//foreach ring in Poly1
-	// foreach ring in Poly2
-	//   if intersect, return 0
+	/*
+	 * foreach ring in Poly1
+	 * foreach ring in Poly2
+	 *   if intersect, return 0
+	 */
 	for (i=0; i<poly1->nrings; i++)
 	{
-		double dist = distance2d_ptarray_poly(poly1->rings[i], poly2);
-		if (i) mindist = LW_MIN(mindist, dist);
-		else mindist = dist;
+		int j;
+		for (j=0; j<poly2->nrings; j++)
+		{
+			double d = distance2d_ptarray_ptarray(poly1->rings[i],
+				poly2->rings[j]);
+			if ( d <= 0 ) return 0.0;
+
+			if (i) mindist = LW_MIN(mindist, d);
+			else mindist = d;
+		}
 
 #ifdef PGIS_DEBUG
-		lwnotice("  ring%d dist: %f, mindist: %f", i, dist, mindist);
+		lwnotice("  ring%d dist: %f, mindist: %f", i, d, mindist);
 #endif
 
-		if ( mindist <= 0 ) return 0.0; // intersection
 	}
 
-	// otherwise return closest approach of rings (no intersection)
+	/* otherwise return closest approach of rings (no intersection) */
 	return mindist;
 
 }
@@ -481,7 +513,7 @@ double distance2d_line_poly(LWLINE *line, LWPOLY *poly)
 }
 
 
-//find the 2d length of the given POINTARRAY (even if it's 3d)
+/*find the 2d length of the given POINTARRAY (even if it's 3d) */
 double lwgeom_pointarray_length2d(POINTARRAY *pts)
 {
 	double dist = 0.0;
@@ -492,9 +524,7 @@ double lwgeom_pointarray_length2d(POINTARRAY *pts)
 	if ( pts->npoints < 2 ) return 0.0;
 	for (i=0; i<pts->npoints-1;i++)
 	{
-		//POINT2D *frm = (POINT2D *)getPoint(pts, i);
 		getPoint2d_p(pts, i, &frm);
-		//POINT2D *to = (POINT2D *)getPoint(pts, i+1);
 		getPoint2d_p(pts, i+1, &to);
 		dist += sqrt( ( (frm.x - to.x)*(frm.x - to.x) )  +
 				((frm.y - to.y)*(frm.y - to.y) ) );
@@ -502,7 +532,10 @@ double lwgeom_pointarray_length2d(POINTARRAY *pts)
 	return dist;
 }
 
-//find the 3d/2d length of the given POINTARRAY (depending on its dimensions)
+/*
+ * Find the 3d/2d length of the given POINTARRAY
+ * (depending on its dimensions)
+ */
 double
 lwgeom_pointarray_length(POINTARRAY *pts)
 {
@@ -513,13 +546,11 @@ lwgeom_pointarray_length(POINTARRAY *pts)
 
 	if ( pts->npoints < 2 ) return 0.0;
 
-	// compute 2d length if 3d is not available
+	/* compute 2d length if 3d is not available */
 	if ( ! TYPE_HASZ(pts->dims) ) return lwgeom_pointarray_length2d(pts);
 
 	for (i=0; i<pts->npoints-1;i++)
 	{
-		//POINT3DZ *frm = (POINT3DZ *)getPoint(pts, i);
-		//POINT3DZ *to = (POINT3DZ *)getPoint(pts, i+1);
 		getPoint3dz_p(pts, i, &frm);
 		getPoint3dz_p(pts, i+1, &to);
 		dist += sqrt( ( (frm.x - to.x)*(frm.x - to.x) )  +
@@ -530,9 +561,12 @@ lwgeom_pointarray_length(POINTARRAY *pts)
 	return dist;
 }
 
-//find the area of the outer ring - sum (area of inner rings)
-// Could use a more numerically stable calculator...
-double lwgeom_polygon_area(LWPOLY *poly)
+/*
+ * Find the area of the outer ring - sum (area of inner rings).
+ * Could use a more numerically stable calculator...
+ */
+double
+lwgeom_polygon_area(LWPOLY *poly)
 {
 	double poly_area=0.0;
 	int i;
@@ -549,21 +583,23 @@ double lwgeom_polygon_area(LWPOLY *poly)
 		POINTARRAY *ring = poly->rings[i];
 		double ringarea = 0.0;
 
-//lwnotice(" rings %d has %d points", i, ring->npoints);
+#if PGIS_DEBUG > 1
+lwnotice(" rings %d has %d points", i, ring->npoints);
+#endif
 		for (j=0; j<ring->npoints-1; j++)
     		{
-			//POINT2D *p1 = (POINT2D *)getPoint(ring, j);
-			//POINT2D *p2 = (POINT2D *)getPoint(ring, j+1);
 			getPoint2d_p(ring, j, &p1);
 			getPoint2d_p(ring, j+1, &p2);
 			ringarea += ( p1.x * p2.y ) - ( p1.y * p2.x );
 		}
 
 		ringarea  /= 2.0;
-//lwnotice(" ring 1 has area %lf",ringarea);
+#if PGIS_DEBUG > 1
+lwnotice(" ring 1 has area %lf",ringarea);
+#endif
 		ringarea  = fabs(ringarea);
-		if (i != 0)	//outer
-			ringarea  = -1.0*ringarea ; // its a hole
+		if (i != 0)	/*outer */
+			ringarea  = -1.0*ringarea ; /* its a hole */
 
 		poly_area += ringarea;
 	}
@@ -571,14 +607,18 @@ double lwgeom_polygon_area(LWPOLY *poly)
 	return poly_area;
 }
 
-// Compute the sum of polygon rings length.
-// Could use a more numerically stable calculator...
+/*
+ * Compute the sum of polygon rings length.
+ * Could use a more numerically stable calculator...
+ */
 double lwgeom_polygon_perimeter(LWPOLY *poly)
 {
 	double result=0.0;
 	int i;
 
-//lwnotice("in lwgeom_polygon_perimeter (%d rings)", poly->nrings);
+#ifdef PGIS_DEBUG
+lwnotice("in lwgeom_polygon_perimeter (%d rings)", poly->nrings);
+#endif
 
 	for (i=0; i<poly->nrings; i++)
 		result += lwgeom_pointarray_length(poly->rings[i]);
@@ -586,14 +626,18 @@ double lwgeom_polygon_perimeter(LWPOLY *poly)
 	return result;
 }
 
-// Compute the sum of polygon rings length (forcing 2d computation).
-// Could use a more numerically stable calculator...
+/*
+ * Compute the sum of polygon rings length (forcing 2d computation).
+ * Could use a more numerically stable calculator...
+ */
 double lwgeom_polygon_perimeter2d(LWPOLY *poly)
 {
 	double result=0.0;
 	int i;
 
-//lwnotice("in lwgeom_polygon_perimeter (%d rings)", poly->nrings);
+#ifdef PGIS_DEBUG
+lwnotice("in lwgeom_polygon_perimeter (%d rings)", poly->nrings);
+#endif
 
 	for (i=0; i<poly->nrings; i++)
 		result += lwgeom_pointarray_length2d(poly->rings[i]);
@@ -617,11 +661,11 @@ lwgeom_mindistance2d_recursive(uchar *lw1, uchar *lw2)
 		int t1 = lwgeom_getType(g1[0]);
 		double dist=0;
 
-		// it's a multitype... recurse
+		/* it's a multitype... recurse */
 		if ( t1 >= 4 )
 		{
 			dist = lwgeom_mindistance2d_recursive(g1, lw2);
-			if ( dist == 0 ) return 0.0; // can't be closer
+			if ( dist == 0 ) return 0.0; /* can't be closer */
 			if ( mindist == -1 ) mindist = dist;
 			else mindist = LW_MIN(dist, mindist);
 			continue;
@@ -704,7 +748,7 @@ lwgeom_mindistance2d_recursive(uchar *lw1, uchar *lw2)
 					);
 				}
 			}
-			else // it's a multitype... recurse
+			else /* it's a multitype... recurse */
 			{
 				dist = lwgeom_mindistance2d_recursive(g1, g2);
 			}
@@ -718,7 +762,7 @@ lwgeom_mindistance2d_recursive(uchar *lw1, uchar *lw2)
 #endif
 
 
-			if (mindist <= 0.0) return 0.0; // can't be closer
+			if (mindist <= 0.0) return 0.0; /* can't be closer */
 
 		}
 
@@ -742,3 +786,56 @@ lwgeom_pt_inside_circle(POINT2D *p, double cx, double cy, double rad)
 
 }
 
+/*
+ * Compute the azimuth of segment AB in radians.
+ * Return 0 on exception (same point), 1 otherwise.
+ */
+int
+azimuth_pt_pt(POINT2D *A, POINT2D *B, double *d)
+{
+	if ( A->x == B->x )
+	{
+		if ( A->y < B->y ) *d=0.0;
+		else if ( A->y > B->y ) *d=M_PI; 
+		else return 0;
+		return 1;
+	}
+
+	if ( A->y == B->y )
+	{
+		if ( A->x < B->x ) *d=M_PI/2; 
+		else if ( A->x > B->x ) *d=M_PI+(M_PI/2);
+		else return 0;
+		return 1;
+	}
+
+	if ( A->x < B->x )
+	{
+		if ( A->y < B->y )
+		{
+			*d=atan(fabs(A->x - B->x) / fabs(A->y - B->y) );
+		}
+		else /* ( A->y > B->y )  - equality case handled above */
+		{
+			*d=atan(fabs(A->y - B->y) / fabs(A->x - B->x) )
+				+ (M_PI/2);
+		}
+	}
+
+	else /* ( A->x > B->x ) - equality case handled above */
+	{
+		if ( A->y > B->y )
+		{
+			*d=atan(fabs(A->x - B->x) / fabs(A->y - B->y) )
+				+ M_PI;
+		}
+		else /* ( A->y < B->y )  - equality case handled above */
+		{
+			*d=atan(fabs(A->y - B->y) / fabs(A->x - B->x) )
+				+ (M_PI+(M_PI/2));
+		}
+	}
+
+	return 1;
+}
+
