@@ -85,6 +85,8 @@ select '58', asewkt('MULTILINESTRING((0 0, 1 1),(0 0, 1 1, 2 2,) )'::GEOMETRY);
 select '59',asewkt('POINT(1 2 3, 4 5 6)'::GEOMETRY);
 select '60',asewkt('POINT(1 2 3 4 5 6 7)'::GEOMETRY);
 select '61',asewkt('LINESTRING(1 1)'::GEOMETRY);
+--select '62',replace(asewkt('POINT( 1e700 0)'::GEOMETRY), 'Infinity', 'inf');
+--select '63',replace(asewkt('POINT( -1e700 0)'::GEOMETRY), 'Infinity', 'inf');
 select '62',asewkt('POINT( 1e700 0)'::GEOMETRY);
 select '63',asewkt('POINT( -1e700 0)'::GEOMETRY);
 select '64',asewkt('MULTIPOINT(1 1, 2 2'::GEOMETRY);
@@ -192,8 +194,6 @@ select '118', length3d('MULTILINESTRING((0 0 0, 1 1 1),(0 0 0, 1 1 1, 2 2 2) )':
 
 ---selection
 
-select '119', within('LINESTRING(-1 -1, -1 101, 101 101, 101 -1)'::GEOMETRY,'BOX3D(0 0, 100 100)'::BOX3D);
-select '120', within('LINESTRING(-1 -1, -1 100, 101 100, 101 -1)'::GEOMETRY,'BOX3D(0 0, 100 100)'::BOX3D);
 
 
 --- TOAST testing
@@ -218,11 +218,14 @@ select '128',a ~ b from TEST;
 
 select '129', mem_size(dropBBOX(a)), mem_size(dropBBOX(b)) from TEST;
 
-select '130', geosnoop('POLYGON((0 0, 1 1, 0 0))');
-
 select '131', X('POINT(1 2)');
 select '132', Y('POINT(1 2)');
 select '133', Z('POINT(1 2)');
+select '133a', Z('POINT(1 2 3)');
+select '133b', Z('POINTM(1 2 3)');
+select '133c', M('POINT(1 2)');
+select '133d', M('POINTM(1 2 4)');
+select '133e', M('POINT(1 2 4)');
 
 select '134', distance('POINT(1 2)', 'POINT(1 2)');
 select '135', distance('POINT(5 0)', 'POINT(10 12)');
@@ -237,3 +240,18 @@ select '140', asewkt(multi(setsrid('POINT(2 2)'::geometry, 3)));
 select '141', asewkt(multi(setsrid('LINESTRING(2 2, 3 3)'::geometry, 4)));
 select '142', asewkt(multi(setsrid('LINESTRING(2 2, 3 3)'::geometry, 5)));
 select '143', asewkt(multi(setsrid('POLYGON((0 0, 1 0, 1 1, 0 1, 0 0))'::geometry, 6)));
+
+select '144', asewkt(force_3dm('POINT(1 2 3)'));
+select '145', asewkt(force_3dz('POINTM(1 2 3)'));
+select '146', asewkt(force_4d('POINTM(1 2 3)'));
+select '147', asewkt(force_4d('POINT(1 2 3)'));
+
+select '148', astext(segmentize('LINESTRING(0 0, 10 0)', 5));
+
+
+select '150', asewkt(force_collection(setsrid('POLYGON((0 0, 1 0, 1 1, 0 1, 0 0))'::geometry, 6)));
+
+select '151', geom_accum(NULL, NULL);
+
+-- Drop test table
+DROP table test;
